/*******************************************************************************
 * @note Copyright (C) 2019 Shanghai Panchip Microelectronics Co., Ltd.
 * All rights reserved.
 *
 * @file driver_delay.c
 * @brief 
 *
 * @history - V0.01, 2019-05-27, caixiaohai, first implementation.
*******************************************************************************/
#include "driver_delay.h"

__IO uint32_t usTick;

/**
  * @brief This function configures the systick is 100us,
  *        The systick is configured  to have 1ms time base with a dedicated 
  *        Tick interrupt priority.
  * @note In the default implementation, SysTick timer is the source of time base. 
  *       It is used to generate interrupts at regular time intervals. 
  *       Care must be taken if delay_ms() is called from a peripheral ISR process, 
  *       The SysTick interrupt must have higher priority (numerically lower)
  *       than the peripheral interrupt. Otherwise the caller ISR process will be blocked.
  *       The function is declared as __weak  to be overwritten  in case of other
  *       implementation  in user file. 
  */
void HAL_InitTick(void)
{
    SysTick->CTRL=0;                    
    NVIC_SetPriority(SysTick_IRQn, (1UL<<__NVIC_PRIO_BITS)-1UL);        /* set Priority for systick Interrupt */
    SysTick->LOAD=SystemCoreClock/10000;                                 /* set reload register */
    SysTick->VAL =0UL;                                                  /* reset systick Counter value */
    SysTick->CTRL = SysTick_CTRL_CLKSOURCE_Msk  |
                    SysTick_CTRL_TICKINT_Msk    |
                    SysTick_CTRL_ENABLE_Msk;                            /* Enable SysTick IRQ and SysTick Timer */
}


/**
  * @brief This function is called to increment  a global variable "msTick"
  *        used as application time base.
  * @note In the default implementation, this variable is incremented each 1ms
  *       in SysTick ISR.
  * @retval None
  */
void HAL_IncTick(void)
{
  usTick += 1;
}

/**
  * @brief Provides a tick value in millisecond.
  * @note This function is declared as __weak to be overwritten in case of other 
  *       implementations in user file.
  * @retval tick value
  */
uint32_t HAL_GetTick(void)
{
  return usTick;
}

/**
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @retval None
  */
void SysTick_Handler(void)
{
	HAL_IncTick();
}

void delay_us(uint32_t us)
{
    SysTick->CTRL = 0;
    SysTick->LOAD =  (us* CyclesPerUs)&SysTick_LOAD_RELOAD_Msk-1;//δֵ  1us
    SysTick->VAL  =  (0x00);//δǰֵLOADȡֵ
    SysTick->CTRL = SysTick->CTRL | SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_ENABLE_Msk;//ʹϵͳʱ ϵͳʱ			
    while((SysTick->CTRL & SysTick_CTRL_COUNTFLAG_Msk)==0);
}
void delay_ms(uint32_t ms)
{
    while(ms--)
    delay_us(1000);
}


