/*******************************************************************************
 * @note Copyright (C) 2019 Shanghai Panchip Microelectronics Co., Ltd.
 * All rights reserved.
 *
 * @file driver_lowpower.c
 * @brief 
 *
 * @history - V0.01, 2020-01-16, yangyaming, first implementation.
*******************************************************************************/
#include "driver_lowpower.h"


/*
  DeepSleep Mode 32K唤醒：功耗较高，50ma左右，芯片不会复位，唤醒后，会接着进入低功耗前的地方继续执行，会产生SLEEP_IRQHandler中断；
  Standby Ram retention 32k唤醒：功耗 3.42ua，芯片会复位，但ram数据会保存，不会产生STANDBY_IRQHandler中断
  

*/

#define CLK_32K_CALIBRATION_VALUE   40000

#define CLK_32K_TRIM        0x00400034ul
#define CLK_16M_TRIM        0x00400038ul
#define BANDGAP_TRIM        0x0040003cul
uint8_t clk_32k_trim,clk_16m_trim,bandgap_trim;

void clk_get_trim(void)
{
    
//    SYS->P1_MFP &= ~SYS_MFP_P15_Msk;
//    SYS->P1_MFP |= SYS_MFP_P15_32K_RCO;
//    
//    SYS->P2_MFP &= ~SYS_MFP_P24_Msk;
//    SYS->P2_MFP |= SYS_MFP_P24_16M_RCO;

    clk_32k_trim = flash_ReadInfo(CLK_32K_TRIM);
	ANAC->LP_ANA_CTRL1 &= ~RC_I_TUNE_Msk;
	ANAC->LP_ANA_CTRL1 |= (clk_32k_trim << 16);
	CLK_Wait3vSyncReady();
    clk_16m_trim = flash_ReadInfo(CLK_16M_TRIM);
	ANAC->ANA_RC_DPLL_CTRL &= ~RC_16M_FRQ_Msk;
	ANAC->ANA_RC_DPLL_CTRL |= clk_16m_trim;
	CLK_Wait3vSyncReady();
    
    bandgap_trim = flash_ReadInfo(BANDGAP_TRIM);
    ANAC->LP_ANA_CTRL1 &= ~(0x1f << 8);
    ANAC->LP_ANA_CTRL1 |= (bandgap_trim << 8);
    CLK_Wait3vSyncReady();
}

//#define NO32K

void lowpower_init(void)
{
    CLK_APB2PeriphClockCmd(CLK_APB2Periph_TMR1,ENABLE);
    
    clk_get_trim();
    
    NVIC_EnableIRQ(SLEEP_IRQn);
	NVIC_EnableIRQ(STANDBY_IRQn);
	NVIC_EnableIRQ(TIMER0_IRQn);
    
    
    CLK->CLK_WKUP_0 &= ~(0xFF << 0); //RCH
    CLK->CLK_WKUP_0 |= (0x1 << 0);
    CLK->CLK_WKUP_0 &= ~(0xFF << 8); //RCL
    CLK->CLK_WKUP_0 |= (0x1 << 8);
//		CLK->CLK_WKUP_0 &= ~(0xFF << 16); //XTH
//		CLK->CLK_WKUP_0 |= (0x1 << 16);
    CLK->CLK_WKUP_1 &= ~(0xFF << 0);  //DPLL
    CLK->CLK_WKUP_1 |= (0x2 << 0);
	CLK_Wait3vSyncReady();
    
    // Set RC16M to 16	
//	ANAC->ANA_RC_DPLL_CTRL  &= ~(0xFF);
//	ANAC->ANA_RC_DPLL_CTRL  |=  (0x7a);
//    // HP LDO V control
	ANAC->LP_ANA_CTRL0 &= ~(0x7 << 24);
	ANAC->LP_ANA_CTRL0 |=  (0x2 << 24); //Default 0x3   0x2: 1.507V
	CLK_Wait3vSyncReady();	
//	
//    // LP LDO V control
//	ANAC->LP_ANA_CTRL1 &= ~(0xF << 4);
//	ANAC->LP_ANA_CTRL1 |=  (0x0 << 4); //Default 0xC
//	CLK_Wait3vSyncReady();
}


#if __LOW_POWER_STANDBY_ENABLE != __FALSE
void low_power_standby_enter(void)
{
	ANAC->LP_1P5V_RST |= RESET_EN_Msk;
	CLK_Wait3vSyncReady();
  #if __P56_WAKE_UP_ENABLE != __FALSE
		ANAC->LP_FL_CTRL0 &= ~EXT_WKUP_DISABLE_Msk;
		GPIO_SetMode(P5,BIT6,GPIO_MODE_INPUT);
		GPIO_ENABLE_DIGITAL_PATH(P5,(1<<6));
		CLK_Wait3vSyncReady();
		#if __P56_GPIO_LEVEL == 0x00
			ANAC->LP_FL_CTRL0 &= ~EXT_WKUP_SEL_Msk;
		#elif __P56_GPIO_LEVEL == 0x01
			ANAC->LP_FL_CTRL0 |= EXT_WKUP_SEL_Msk;
		#endif
	#endif
    #if __32K_WAKE_UP_ENABLE != __FALSE
        LP_SetSleepTime(ANAC,CLK_32K_CALIBRATION_VALUE*5);
        CLK_Wait3vSyncReady();
    #endif
    //切换系统时钟为16M RC
    CLK_SYSCLKConfig(CLK_AHB_CLKSEL_16M_RC);
    
    LP_SetStandbyConfig(ANAC,__LOW_POWER_STANDBY_MODE_SRAM_DATA_IS_RETENTION);
    CLK_Wait3vSyncReady();
    
    LP_RequestSleep(ANAC);
    CLK_Wait3vSyncReady();
}


#elif __LOW_POWER_DEEP_SLEEP_ENABLE != __FALSE
void low_power_deep_sleep_enter(void)
{
    #if __LOW_POWER_DEEP_SLEEP_WAKE_UP_BY_GPIO != FALSE
        ANAC->LP_FL_CTRL0 &= ~EXT_WKUP_DISABLE_Msk;
        #if __LOW_POWER_DEEP_SLEEP_MODE_WAKE_UP_BY_GPIO_MODE == 0x00
            GPIO_SetMode(P5,BIT6,GPIO_MODE_INPUT);
            GPIO_ENABLE_DIGITAL_PATH(P5,(1<<6));
            CLK_Wait3vSyncReady();
            #if __LOW_POWER_DEEP_SLEEP_MODE_WAKE_UP_BY_GPIO_LEVEL == 0x00
                ANAC->LP_FL_CTRL0 &= ~EXT_WKUP_SEL_Msk;
            #elif __LOW_POWER_DEEP_SLEEP_MODE_WAKE_UP_BY_GPIO_LEVEL == 0x01
                ANAC->LP_FL_CTRL0 |= EXT_WKUP_SEL_Msk;
            #endif
        #elif __LOW_POWER_DEEP_SLEEP_MODE_WAKE_UP_BY_GPIO_MODE == 0x01
            if(__GPIO_WAKE_UP_PORT == 0){
                NVIC_EnableIRQ(GPIO0_IRQn);
            }else if(__GPIO_WAKE_UP_PORT == 1){    
                NVIC_EnableIRQ(GPIO1_IRQn);
            }else if(__GPIO_WAKE_UP_PORT == 2){    
                NVIC_EnableIRQ(GPIO2_IRQn);
            }else if(__GPIO_WAKE_UP_PORT == 3){    
                NVIC_EnableIRQ(GPIO3_IRQn);
            }else if(__GPIO_WAKE_UP_PORT == 4){    
                NVIC_EnableIRQ(GPIO4_IRQn);
            }else if(__GPIO_WAKE_UP_PORT == 5){    
                NVIC_EnableIRQ(GPIO5_IRQn);
            }
            if((__GPIO_WAKE_UP_PORT == 4)&& (__GPIO_WAKE_UP_PIN == 6)){
                SYS->P4_MFP &= ~SYS_MFP_P46_Msk;
                SYS->P4_MFP |= SYS_MFP_P46_GPIO;
            }
            if((__GPIO_WAKE_UP_PORT == 4)&& (__GPIO_WAKE_UP_PIN == 7)){
                SYS->P4_MFP &= ~SYS_MFP_P47_Msk;
                SYS->P4_MFP |= SYS_MFP_P47_GPIO;
            }
            #define __PORT__(n)       ((GPIO_T *) (GPIO0_BASE + (0x00000000 | ((n) << 6))))
            #define __PIN__(n)        (1 << (n))
            GPIO_SetMode(__PORT__(__GPIO_WAKE_UP_PORT),__PIN__(__GPIO_WAKE_UP_PIN),GPIO_MODE_INPUT);
            #if __LOW_POWER_DEEP_SLEEP_MODE_WAKE_UP_BY_GPIO_LEVEL == 0x00
								printf("enable gpio interrupt\r\n");
                GPIO_EnableInt(__PORT__(__GPIO_WAKE_UP_PORT),__GPIO_WAKE_UP_PIN,GPIO_INT_FALLING);
            #elif __LOW_POWER_DEEP_SLEEP_MODE_WAKE_UP_BY_GPIO_LEVEL == 0x01
                GPIO_EnableInt(__PORT__(__GPIO_WAKE_UP_PORT),__GPIO_WAKE_UP_PIN,GPIO_INT_RISING);
            #endif
            
//            GPIO_SetMode(P2,BIT3,GPIO_MODE_INPUT);
//            GPIO_EnableInt(P2,3,GPIO_INT_LOW);
        #endif
		#endif
    #if __LOW_POWER_DEEP_SLEEP_WAKE_UP_BY_32k  != FALSE
//        ANAC->LP_FL_CTRL0 |= EXT_WKUP_DISABLE_Msk;
        ANAC->LP_FL_CTRL0 |= SLEEP_CNT_EN_Msk;
        LP_SetSleepTime(ANAC,CLK_32K_CALIBRATION_VALUE*__LOW_POWER_SLEEP_TIME);
        CLK_Wait3vSyncReady();
    #endif                       

    //wakeup reset mode disable
    LP_DisableDigitalRst(ANAC);		
    CLK_Wait3vSyncReady();
    //enable small ldo    
		LP_EnableLpLdo(ANAC);
		CLK_Wait3vSyncReady();
       
    LP_SetDeepSleepConfig(ANAC);
    CLK_Wait3vSyncReady(); 
    
    __WFI();
	
	
}

#endif

extern uint8_t wake_up_32k_flag;
extern uint8_t wake_up_p56_flag;
 uint8_t wake_up_other_gpio_flag;
//extern uint8_t standby_test;
void SLEEP_IRQHandler(void)
{
    if(ANAC->LP_INT_CTRL & P56_WK_FLG_Msk){
        
        LP_ClearWakeFlag(ANAC,P56_WK_FLG_Msk);
        wake_up_p56_flag = 0x01;
    }else if(ANAC->LP_INT_CTRL & TMR_32K_WK_FLG_Msk){
       
        LP_ClearWakeFlag(ANAC,TMR_32K_WK_FLG_Msk);
        wake_up_32k_flag = 0x01;
    }
    LP_ClearWakeFlag(ANAC,SLEEP_INT_FLG_Msk);
	CLK_Wait3vSyncReady();
	
}

void TMR1_IRQHandler(void)
{
    SYS_TEST("Timer 1 interrupt \r\n");
    
    /* clear timer interrupt flag */
    TIMER_ClearIntFlag(TIMER1);
}

void STANDBY_IRQHandler(void)
{
  SYS_TEST("standby interrupt\r\n");
	LP_ClearWakeFlag(ANAC,STANDBY_INT_FLG_Msk);
    //CLK_Wait3vSyncReady();
}
#if __LOW_POWER_DEEP_SLEEP_ENABLE != __FALSE
void gpio_irq_handler(void)
{
    
    #define __PORT__(n)       ((GPIO_T *) (GPIO0_BASE + (0x00000000 | ((n) << 6))))
    #define __PIN__(n)        (1 << (n))
    if(__PORT__(__GPIO_WAKE_UP_PORT)->INTSRC & __PIN__(__GPIO_WAKE_UP_PIN)){
//        GPIO_DisableInt(__PORT__(__GPIO_WAKE_UP_PORT), __GPIO_WAKE_UP_PIN);
        __PORT__(__GPIO_WAKE_UP_PORT)->INTSRC = __PIN__(__GPIO_WAKE_UP_PIN);
        printf("GPIO interrupt \r\n");
        wake_up_other_gpio_flag = 0x01;
		
    }else{
        __PORT__(__GPIO_WAKE_UP_PORT)->INTSRC = __PORT__(__GPIO_WAKE_UP_PORT)->INTSRC;
    }
}
void GPIO0_IRQHandler(void)
{
    gpio_irq_handler();
}
void GPIO1_IRQHandler(void)
{
    gpio_irq_handler();
}

void GPIO2_IRQHandler(void)
{
    gpio_irq_handler();
    
}

void GPIO3_IRQHandler(void)
{
    gpio_irq_handler();
}

void GPIO4_IRQHandler(void)
{
    gpio_irq_handler();
}

void GPIO5_IRQHandler(void)
{
    gpio_irq_handler();
}

#endif

