/*******************************************************************************
 * @note Copyright (C) 2019 Shanghai Panchip Microelectronics Co., Ltd.
 * All rights reserved.
 *
 * @file driver_rf.c
 * @brief 
 *
 * @history - V0.01, 2019-06-24, caixiaohai, first implementation.
*******************************************************************************/
#include "driver_rf.h"
#include "driver_delay.h"

uint8_t rf_tx_payload[TX_PAYLOAD_LEN];
uint8_t rf_rx_payload[RX_PAYLOAD_LEN];

uint8_t tx_addr[5] = TX_ADDR_DEF;


uint8_t EnhanceMode;

volatile uint32_t g_rx_cnt = 0;

volatile uint8_t	g_tx_flag = 0;
volatile uint8_t	g_rx_flag = 0;
volatile uint8_t	g_tx_rx_flag = 0;
volatile uint32_t	g_discard_pkts=0;

void RF_IRQHandler(void)
{
	uint32_t rx_length = 0;
	
	/* In enhanced mode, the master sends the payload and receives the ack with payload*/
	if((RF->IRQ_STATUS & IRQ_RX_TX) == IRQ_RX_TX){
		 RF_ReadBuf(R_RX_PAYLOAD,rf_rx_payload, RF_GetRxLength());
		 g_tx_rx_flag = 1;
         RF_ClearFIFO();	
	}
	/* new data arrives in RX FIFO*/
	else if(((RF->IRQ_STATUS) & IRQ_RX_DR) == IRQ_RX_DR)
	{
		rx_length = RF_GetRxLength();
		RF_ReadBuf(R_RX_PAYLOAD,rf_rx_payload, rx_length);
		g_rx_flag = 1;
		g_rx_cnt++;
		
		#if((FEATURE_MODE&ACK_WITH_PAYLOAD_MSK) == ACK_WITH_PAYLOAD_MSK)
            RF_WriteBuf(W_ACK_PAYLOAD,rf_rx_payload,rx_length);
		#endif
		RF->RF_CMD = FLUSH_RX;
	}
	/**In enhanced mode	: sends the payload and receives ack
		*In busrt mode		:	payload sent successfully				*/
	else if(((RF->IRQ_STATUS) & IRQ_TX_DS)== IRQ_TX_DS)
	{
		RF->RF_CMD = FLUSH_TX;	
		g_tx_flag = 1;
	}
	/* reaching the maximum transmission times */
	else if(((RF->IRQ_STATUS) & IRQ_MAX_RT)== IRQ_MAX_RT)
	{
		RF->RF_CMD = FLUSH_TX;
		

		#if((FEATURE_MODE&ACK_WITH_PAYLOAD_MSK) == ACK_WITH_PAYLOAD_MSK)
		g_tx_rx_flag = 1;
		#else
		g_tx_flag = 1;
		#endif
		g_discard_pkts++;
	}
	
	RF->IRQ_STATUS = 0x70;	
}

void RF_PackData(uint8_t *pBuf, uint8_t length)
{
    uint8_t i; 
    
    for(i = 0;i < length; i++){
        pBuf[i] = 0x55;
    }
}

void rf_tx_busrt_interrupte_test(void)
{
	volatile uint32_t cnt = 0;
	RF_Init();
	PowerSet();
	NVIC_EnableIRQ(RF_IRQn);
	RF_TxMode();
	RF_SetChannel(RF_CHANNEL);
	printf("|  rf init sucess                                           |\n");

	RF_PackData(rf_tx_payload,sizeof(rf_tx_payload));
	RF_WriteBuf(W_TX_PAYLOAD,rf_tx_payload, sizeof(rf_tx_payload));
	
	while(1){
		for(cnt = 1; cnt < 1000 ; cnt++)
		{
			while(!g_tx_flag);
			g_tx_flag = 0;
			printf("Tx OK,Send %d packets. \n",cnt);
//			delay_us(1000);
			RF_WriteBuf(W_TX_PAYLOAD,rf_tx_payload, sizeof(rf_tx_payload));
		} 
		printf("|  Please press any char to continue                        |\n");
		getchar();		
	}
}

void rf_rx_busrt_interrupte_test(void)
{	
	RF_Init();
	NVIC_EnableIRQ(RF_IRQn);	
	RF_RxMode();
	RF_SetChannel(RF_CHANNEL);
	printf("|  rf init sucess                                           |\n");
	
	while(1){
		delay_ms(500);
		printf("rx pkts = %d\n",g_rx_cnt);
	}
}


void rf_tx_enhance_it_test(void)
{
	volatile uint32_t cnt = 0;
	
	RF_Init();
	PowerSet();
	NVIC_EnableIRQ(RF_IRQn);	
	RF_TxMode();
	RF_SetChannel(RF_CHANNEL);
	printf("|  rf init sucess                                           |\n");
	
	RF_PackData(rf_tx_payload,sizeof(rf_tx_payload));
	#if(NOACK_FLAG == NOACK_MSK)
	RF_WriteBuf(W_TX_PAYLOAD_NOACK,rf_tx_payload, sizeof(rf_tx_payload));
	#else
	RF_WriteBuf(W_TX_PAYLOAD,rf_tx_payload, sizeof(rf_tx_payload));
	#endif
	
	while(1)
	{			
		if(cnt<999)
		{
			#if ACK_WITH_PAYLOAD_FLAG==0
			if(g_tx_flag)
			{
				cnt++;
				printf("Tx OK,Send %d packets. \n",cnt);
			
				#if DYN_FLAG==DYN_MSK
				
						#if(NOACK_FLAG == NOACK_MSK)
						if(cnt % 2 == 0)
						{
								printf("Send %d bytes .\n",32);
								RF_WriteBuf(W_TX_PAYLOAD_NOACK,rf_tx_payload, 32);
						}
						else
						{
								printf("Send %d bytes .\n",2);
								RF_WriteBuf(W_TX_PAYLOAD_NOACK,rf_tx_payload, 2);
						}
						#else
						if(cnt % 2 == 0)
						{
								printf("Send %d bytes .\n",32);
								RF_WriteBuf(W_TX_PAYLOAD,rf_tx_payload, 32);
						}
						else
						{
								printf("Send %d bytes .\n",2);
								RF_WriteBuf(W_TX_PAYLOAD,rf_tx_payload, 2);
						}
						#endif
						
				#else
						
						#if(NOACK_FLAG == NOACK_MSK)
						RF_WriteBuf(W_TX_PAYLOAD_NOACK,rf_tx_payload, sizeof(rf_tx_payload));
						#else
						RF_WriteBuf(W_TX_PAYLOAD,rf_tx_payload, sizeof(rf_tx_payload));
						#endif
						
				#endif
				g_tx_flag=0;
			}
			#else	
			
			if(g_tx_rx_flag)
			{
				cnt++;
				printf("Tx/Rx OK ,Send %d packets.\n",cnt);
				#if DYN_FLAG==DYN_MSK
					if(cnt % 2 == 0)
					{
							printf("Send %d bytes .\n",32);
							RF_WriteBuf(W_TX_PAYLOAD,rf_tx_payload, 32);
					}
					else
					{
							printf("Send %d bytes .\n",2);
							RF_WriteBuf(W_TX_PAYLOAD,rf_tx_payload, 2);
					}
				#else
					RF_WriteBuf(W_TX_PAYLOAD,rf_tx_payload, sizeof(rf_tx_payload));
				#endif
				g_tx_rx_flag=0;
				
			}
			
			#endif	//ACK_WITH_PAYLOAD
			
		}
		else
		{
			printf("|  send 1000 payload,discard %d byte                        |\n",g_discard_pkts);
			printf("|  Please press any char to continue                        |\n");
			cnt=0;
			g_discard_pkts=0;
			getchar();
		}
	}
}




void rf_rx_enhance_it_test(void)
{
	RF_Init();
	NVIC_EnableIRQ(RF_IRQn);	
	RF_RxMode();
	RF_SetChannel(RF_CHANNEL);
	printf("|  rf init sucess                                           |\n");
	
	while(1){
		delay_ms(500);
		printf("rx pkts = %d\n",g_rx_cnt);
	}
}

