;/*******************************************************************************
; * @note Copyright (C) 2019 Shanghai Panchip Microelectronics Co., Ltd.
; * All rights reserved.
; *
; * @file main.c
; * @brief 
; *
; * @history - V0.01, 2019-03-18, yangyaming, first implementation.
;*******************************************************************************/
    IF :LNOT: :DEF: Stack_Size
Stack_Size      EQU     0x00000600
    ENDIF

                AREA    STACK, NOINIT, READWRITE, ALIGN=3
Stack_Mem       SPACE   Stack_Size
__initial_sp


; <h> Heap Configuration
;   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>
    IF :LNOT: :DEF: Heap_Size
Heap_Size       EQU     0x00000000
    ENDIF
                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base
Heap_Mem        SPACE   Heap_Size
__heap_limit


                PRESERVE8
                THUMB


; Vector Table Mapped to Address 0 at Reset
                AREA    RESET, DATA, READONLY
                EXPORT  __Vectors

__Vectors       DCD     __initial_sp              ; Top of Stack
                DCD     Reset_Handler             ; Reset Handler
                DCD     NMI_Handler               ; NMI Handler
                DCD     HardFault_Handler         ; Hard Fault Handler
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     SVC_Handler               ; SVCall Handler
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     PendSV_Handler            ; PendSV Handler
                DCD     SysTick_Handler           ; SysTick Handler

                ; External Interrupts
                                                  ; maximum of 32 External Interrupts are possible
                ; External Interrupts  ;AHB1--APB1
	            DCD     I2C0_IRQHandler            ; 0,         
                DCD     SPI0_IRQHandler            ; 1,         
                DCD     0                          ; 2, Reserved        
                DCD     UART0_IRQHandler           ; 3,        
				DCD     PWM_IRQHandler             ; 4,  
				DCD     ADC_IRQHandler             ; 5,        				
				DCD     WDT_IRQHandler             ; 6,        
				DCD     WWDT_IRQHandler            ; 7,       
                DCD     TIMER0_IRQHandler          ; 8,        

                ; External Interrupts  ;AHB1--APB2
	            DCD     I2C1_IRQHandler            ; 9,         
                DCD     SPI1_IRQHandler            ; 10,         
                DCD     0                          ; 11, Reserved       
                DCD     UART1_IRQHandler           ; 12,        
                DCD     TIMER1_IRQHandler          ; 13,        
                DCD     TIMER2_IRQHandler          ; 14,        
                DCD     RF_IRQHandler              ; 15, 
                DCD     UART2_IRQHandler           ; 16,        
                DCD     UART3_IRQHandler           ; 17,        
                				
                ; External Interrupts  ;AHB1
                DCD     GPIO0_IRQHandler           ; 18,        
                DCD     GPIO1_IRQHandler           ; 19,        
                DCD     GPIO2_IRQHandler           ; 20,        
                DCD     GPIO3_IRQHandler           ; 21,        
                DCD     GPIO4_IRQHandler           ; 22,        
                DCD     GPIO5_IRQHandler           ; 23,
                DCD     EXT0_IRQHandler            ; 24,
                DCD     EXT1_IRQHandler            ; 25,
                DCD     EXT2_IRQHandler            ; 26,
				
                ; External Interrupts  ;AHB2
	            DCD     DMA_IRQHandler             ; 27,         
				DCD     SLEEP_IRQHandler           ; 28, 
				DCD     STANDBY_IRQHandler         ; 29, 
				DCD     BOD_IRQHandler             ; 30, 					
                DCD     UART4_IRQHandler           ; 31,        
				
                
                AREA    |.text|, CODE, READONLY,  ALIGN=9
                
                
                
; Reset Handler 
             
                ENTRY
                
Reset_Handler   PROC
                EXPORT  Reset_Handler             [WEAK]
				IMPORT  __main 
				IMPORT	SystemInit
				
				LDR		R0,=SystemInit
				BLX		R0

				LDR		R0,=__main
				BX		R0
                ENDP

                
                
; Dummy Exception Handlers (infinite loops which can be modified)                
                
NMI_Handler     PROC
                EXPORT  NMI_Handler               [WEAK]
                B       .
                ENDP
HardFault_Handler\
                PROC
                EXPORT  HardFault_Handler         [WEAK]
                B       .
                ENDP
SVC_Handler     PROC
                EXPORT  SVC_Handler               [WEAK]
                B       .
                ENDP
PendSV_Handler  PROC
                EXPORT  PendSV_Handler            [WEAK]
                B       .
                ENDP
SysTick_Handler	PROC
				EXPORT  SysTick_Handler           [WEAK]
				;IMPORT  timer_handler
				;LDR     R0, = timer_handler			; get the address of the pointer to the int0_handler into R0
				;BX      R0							; branch to the int0_handler
				B       .
				ENDP

Default_Handler PROC
                EXPORT  I2C0_IRQHandler            [WEAK]                                        
                EXPORT  SPI0_IRQHandler            [WEAK]                      
                ; EXPORT  SPIS0_IRQHandler         [WEAK]         
                EXPORT  UART0_IRQHandler           [WEAK]                     
                EXPORT  PWM_IRQHandler             [WEAK]                                         
                EXPORT  ADC_IRQHandler             [WEAK]                                            
                EXPORT  WDT_IRQHandler             [WEAK]                                            
                EXPORT  WWDT_IRQHandler            [WEAK]                                             
                EXPORT  TIMER0_IRQHandler          [WEAK]                                            
                EXPORT  I2C1_IRQHandler            [WEAK]                                            
                EXPORT  SPI1_IRQHandler            [WEAK]                                
                ; EXPORT  SPIS1_IRQHandler         [WEAK]                                   
                EXPORT  UART1_IRQHandler           [WEAK] 				
                EXPORT  TIMER1_IRQHandler          [WEAK] 
				EXPORT  TIMER2_IRQHandler          [WEAK]                           
                EXPORT  RF_IRQHandler              [WEAK] 
                EXPORT  UART2_IRQHandler           [WEAK]                                   
                EXPORT  UART3_IRQHandler           [WEAK]                                   				
                EXPORT  GPIO0_IRQHandler           [WEAK]
                EXPORT  GPIO1_IRQHandler           [WEAK]
                EXPORT  GPIO2_IRQHandler           [WEAK]
                EXPORT  GPIO3_IRQHandler           [WEAK]
                EXPORT  GPIO4_IRQHandler           [WEAK]
                EXPORT  GPIO5_IRQHandler           [WEAK]
                EXPORT  EXT0_IRQHandler            [WEAK]
                EXPORT  EXT1_IRQHandler            [WEAK]
                EXPORT  EXT2_IRQHandler            [WEAK]
                EXPORT  DMA_IRQHandler             [WEAK]
				EXPORT  SLEEP_IRQHandler           [WEAK] 
				EXPORT  STANDBY_IRQHandler         [WEAK]
				EXPORT  BOD_IRQHandler             [WEAK]   ; 30, 					
                EXPORT  UART4_IRQHandler           [WEAK]   ; 34,        
				
					
I2C0_IRQHandler            ; 0,         
SPI0_IRQHandler            ; 1,         
; SPIS0_IRQHandler         ; 2,         
UART0_IRQHandler           ; 3,        
PWM_IRQHandler             ; 4,        
ADC_IRQHandler             ; 5,        
WDT_IRQHandler             ; 6,        
WWDT_IRQHandler            ; 7,       
TIMER0_IRQHandler          ; 8,        

I2C1_IRQHandler            ; 9,         
SPI1_IRQHandler            ; 10,         
; SPIS1_IRQHandler         ; 11,         
UART1_IRQHandler           ; 12,        
TIMER1_IRQHandler          ; 13,        
TIMER2_IRQHandler            ; 14,        
RF_IRQHandler              ; 15,
UART2_IRQHandler           ; 16,        
UART3_IRQHandler           ; 17,        

GPIO0_IRQHandler           ; 18,        
GPIO1_IRQHandler           ; 19,        
GPIO2_IRQHandler           ; 20,        
GPIO3_IRQHandler           ; 21,        
GPIO4_IRQHandler           ; 22,        
GPIO5_IRQHandler           ; 23,
EXT0_IRQHandler            ; 24,
EXT1_IRQHandler            ; 25,
EXT2_IRQHandler            ; 26,
	                      
DMA_IRQHandler             ; 27, 
SLEEP_IRQHandler           ; 28, 
STANDBY_IRQHandler         ; 29,
BOD_IRQHandler             ; 30, 
UART4_IRQHandler           ; 31,        

                B       .
                ENDP


                ALIGN


;User Initial Stack & Heap

                IF      :DEF:__MICROLIB
                
                EXPORT  __initial_sp
                EXPORT  __heap_base
                EXPORT  __heap_limit
                
                ELSE
                
                IMPORT  __use_two_region_memory
                EXPORT  __user_initial_stackheap
__user_initial_stackheap

                LDR     R0, =  Heap_Mem
                LDR     R1, = (Stack_Mem + Stack_Size)
                LDR     R2, = (Heap_Mem +  Heap_Size)
                LDR     R3, = Stack_Mem
                BX      LR

                ALIGN

                ENDIF


                END


