/*******************************************************************************
 * @note Copyright (C) 2019 Shanghai Panchip Microelectronics Co., Ltd.
 * All rights reserved.
 *
 * @file system_pan2025xx.c
 * @brief 
 *
 * @history - V0.01, 2019-03-18, yangyaming, first implementation.
*******************************************************************************/


#include <stdint.h>
#include "pan2025xx.h"



/*----------------------------------------------------------------------------
  Clock Variable definitions
 *----------------------------------------------------------------------------*/
uint32_t __HSI = __HIRC;                 /*!< Factory Default is internal high speed RC 44.2368M divided by 2 */
uint32_t SystemCoreClock;                /*!< System Clock Frequency (Core Clock) */
uint32_t PllClock = __HIRC;              /*!< PLL Output Clock Frequency         */
uint32_t CyclesPerUs;                    /*!< Cycles per micro second */

/**
 *  @brief  Check HIRC clock rate feed to HCLK
 *
 *  @return none
 */

void SystemInit (void)
{

}

/**
  * @brief  This function is used to update the variable SystemCoreClock
  *   and must be called whenever the core clock is changed.
  * @param  None.
  * @retval None.
  */

void SystemCoreClockUpdate (void)
{
    uint32_t core_freq = 0, clk_src = 0;
	uint32_t ahb_div = 0;
    uint32_t pll_clk = 0;

    /* Update PLL Clock */
    pll_clk = CLK_GetPLLClockFreq();
    
    ahb_div = (CLK->CLK_DIV_0 & CLK_CLK_DIV0_AHB_DIV_Msk) >> CLK_CLK_DIV0_AHB_DIV_Pos;
    clk_src = CLK->CLK_AHB_SEL & CLK_CLK_AHB_SEL_Msk;

    if (clk_src == CLK_AHB_CLKSEL_16M_RC)
    {
        /* internal rc clock */
        /*need to tunning rc 16M*/
        core_freq = FREQ_16MHZ;      
    }
    else if (clk_src == CLK_AHB_CLKSEL_16M_HXT)
    {
        /* External crystal clock */
        core_freq = FREQ_16MHZ;       
    }
    else if (clk_src == CLK_AHB_CLKSEL_MCU_DPLL)
    {
        /* mcu dpll output clock */
        core_freq = pll_clk;     
    }
    else
        core_freq = FREQ_16MHZ;      
   
    if(0 == ahb_div)
        SystemCoreClock = core_freq;
    else
    {
        SystemCoreClock = core_freq /(ahb_div+1);
    }
  
    CyclesPerUs = (SystemCoreClock + 500000) / 1000000;
}

#if USE_ASSERT

/**
 * @brief      Assert Error Message
 *
 * @param[in]  file  the source file name
 * @param[in]  line  line number
 *
 * @return     None
 *
 * @details    The function prints the source file name and line number where
 *             the ASSERT_PARAM() error occurs, and then stops in an infinite loop.
 */
void AssertError(uint8_t * file, uint32_t line)
{

    printf("[%s] line %d : wrong parameters.\r\n", file, line);

    /* Infinite loop */
    while(1) ;
}
#endif



