/*******************************************************************************
 * @note Copyright (C) 2020 Shanghai Panchip Microelectronics Co., Ltd.
 * All rights reserved.
 *
 * @file clk.h
 * @brief 
 *
 * @history - V0.01, 2020-03-19, yangyaming, first implementation.
*******************************************************************************/

#ifndef __CLK_H
#define __CLK_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "pan2025xx.h"

 #define FREQ_16MHZ         (16000000)
 #define FREQ_48MHZ         (48000000)
 #define FREQ_57M6HZ        (57600000)
 #define FREQ_72MHZ         (72000000)
// #define FREQ_96MHZ         (96000000)

 //select dpll output
 #define CLK_DPLL_CLKSEL_48M        (0x00)
 #define CLK_DPLL_CLKSEL_57M_6      (0x01)
 #define CLK_DPLL_CLKSEL_72M        (0x02)
 #define CLK_DPLL_CLKSEL_72M_CASE1  (0x03)
    
 /*---------------------------------------------------------------------------------------------------------*/
 /*  Peripheral CLK constant definitions.                                                                          */
 /*---------------------------------------------------------------------------------------------------------*/
 #define CLK_APB_WDTSEL_APB1_DIV2048        0x00000000UL /*!< Setting WDT clock source as APB1/2048 */
 #define CLK_APB_WDTSEL_LIRC                0x00000001UL /*!< Setting WDT clock source as internal 32KHz RC clock */
 
 #define CLK_APB_WWDTSEL_APB1_DIV2048   	0x00000000UL /*!< Setting WWDT clock source as APB1/2048 */ 
 #define CLK_APB_WWDTSEL_LIRC           	0x00000002UL /*!< Setting WWDT clock source as internal RC clock */ 
  
 #define CLK_APB_TMR0SEL_APB1CLK            0x00000000UL /*!< Setting Timer 0 clock source as APB1 */
 #define CLK_APB_TMR0SEL_LIRC               0x00000004UL /*!< Setting Timer 0 clock source as internal 32KHz RC clock */
 #define CLK_APB_TMR0SEL_TM0                0x00000008UL /*!< Setting Timer 0 clock source as external trigger */

 #define CLK_APB_TMR1SEL_APB1CLK            0x00000000UL /*!< Setting Timer 1 clock source as HCLK */
 #define CLK_APB_TMR1SEL_LIRC               0x00010000UL /*!< Setting Timer 1 clock source as internal 32KHz RC clock */
 #define CLK_APB_TMR1SEL_TM1                0x00020030UL /*!< Setting Timer 1 clock source as external trigger */

 #define CLK_APB_TMR2SEL_APB1CLK            0x00000000UL /*!< Setting Timer 2 clock source as HCLK */
 #define CLK_APB_TMR2SEL_LIRC               0x00040000UL /*!< Setting Timer 2 clock source as internal 32KHz RC clock */
 #define CLK_APB_TMR2SEL_TM2                0x00080000UL /*!< Setting Timer 2 clock source as external trigger */

 /*---------------------------------------------------------------------------------------------------------*/
 /*  MCU constant definitions.                                                                          */
 /*---------------------------------------------------------------------------------------------------------*/

 #define CLK_AHB_16M_SRC_CLKSEL_HXT         0x00000001UL /*!< Setting 16M clock source as HXT */ 
 #define CLK_AHB_16M_SRC_CLKSEL_LIRC        0x00000000UL /*!< Setting 16M clock source as LIRC */ 


 #define CLK_AHB_CLKSEL_16M_RC         	0x00000000UL /*!< Setting MCU clock source as 16M RC */ 
 #define CLK_AHB_CLKSEL_16M_HXT        	0x00000001UL /*!< Setting MCU clock source as 16M RC */ 
 #define CLK_AHB_CLKSEL_MCU_DPLL      	0x00000002UL /*!< Setting MCU clock source as DPLL */ 


 /*---------------------------------------------------------------------------------------------------------*/
 /*  CLKDIV constant definitions.                                                                           */
 /*---------------------------------------------------------------------------------------------------------*/
/** @defgroup CLK_System_Clock_Source 
  * @{
  */
#define CLK_SYSCLKSource_HSI             ((uint32_t)0x00000000)
#define CLK_SYSCLKSource_HSE             ((uint32_t)0x00000001)
#define CLK_SYSCLKSource_PLLCLK          ((uint32_t)0x00000002)
#define IS_CLK_SYSCLK_SOURCE(SOURCE) (((SOURCE) == CLK_SYSCLKSource_HSI) || \
                                      ((SOURCE) == CLK_SYSCLKSource_HSE) || \
                                      ((SOURCE) == CLK_SYSCLKSource_PLLCLK))
/**
  * @}
  */ 
/** @defgroup MCU DPLL Divider
  * @{
  */
#define CLK_MCU_DPLL_Div1                  ((uint32_t)0x0001)
#define CLK_MCU_DPLL_Div2                  ((uint32_t)0x0002)
#define CLK_MCU_DPLL_Div4                  ((uint32_t)0x0004)
#define CLK_MCU_DPLL_Div8                  ((uint32_t)0x0008)
#define IS_MCU_DPLL_HCLK(CLK) (((CLK) == CLK_MCU_DPLL_Div1) || ((CLK) == CLK_MCU_DPLL_Div2) || \
                              ((CLK) == CLK_MCU_DPLL_Div4) || ((CLK) == CLK_MCU_DPLL_Div8)
/**
  * @}
  */ 
/** @defgroup AHB Divider
  * @{
  */
#define CLK_AHB_CLK_Div1                  ((uint32_t)0x000100)
#define CLK_AHB_CLK_Div2                  ((uint32_t)0x000200)
#define CLK_AHB_CLK_Div4                  ((uint32_t)0x000400)
#define CLK_AHB_CLK_Div8                  ((uint32_t)0x000800)
#define IS_AHB_CLK(HCLK) (((HCLK) == CLK_AHB_CLK_Div1) || ((HCLK) == CLK_AHB_CLK_Div2) || \
                           ((HCLK) == CLK_AHB_CLK_Div4) || ((HCLK) == CLK_AHB_CLK_Div8)
/**
  * @}
  */ 
/** @defgroup RF DPLL Divider
  * @{
  */
#define CLK_RF_DPLL_Div1                  ((uint32_t)0x00010000)
#define CLK_RF_DPLL_Div2                  ((uint32_t)0x00020000)
#define CLK_RF_DPLL_Div4                  ((uint32_t)0x00040000)
#define CLK_RF_DPLL_Div8                  ((uint32_t)0x00080000)
#define IS_RF_DPLL_CLK(HCLK) (((HCLK) == CLK_RF_DPLL_Div1) || ((HCLK) == CLK_RF_DPLL_Div2) || \
                             ((HCLK) == CLK_RF_DPLL_Div4) || ((HCLK) == CLK_RF_DPLL_Div8)
/**
  * @}
  */ 
/** @defgroup CLK_APB1_Clock_Source
  * @{
  */
#define CLK_HCLK_APB2_Div1                    ((uint32_t)0x0000)
#define CLK_HCLK_APB2_Div2                    ((uint32_t)0x0100)
#define CLK_HCLK_APB2_Div4                    ((uint32_t)0x0200)
#define CLK_HCLK_APB2_Div8                    ((uint32_t)0x0400)
#define CLK_HCLK_APB2_Div16                   ((uint32_t)0x0800)
#define IS_CLK_PCLK2(PCLK) (((PCLK) == CLK_HCLK_APB2_Div1) || ((PCLK) == CLK_HCLK_APB2_Div2) || \
                           ((PCLK) == CLK_HCLK_APB2_Div4) || ((PCLK) == CLK_HCLK_APB2_Div8) || \
                           ((PCLK) == CLK_HCLK_APB2_Div16))
/**
  * @}
  */ 
/** @defgroup CLK_APB2_Clock_Source
  * @{
  */
#define CLK_HCLK_APB1_Div1                    ((uint32_t)0x0000)
#define CLK_HCLK_APB1_Div2                    ((uint32_t)0x0001)
#define CLK_HCLK_APB1_Div4                    ((uint32_t)0x0002)
#define CLK_HCLK_APB1_Div8                    ((uint32_t)0x0004)
#define CLK_HCLK_APB1_Div16                   ((uint32_t)0x0008)
#define IS_CLK_PCLK1(PCLK) (((PCLK) == CLK_HCLK_APB1_Div1) || ((PCLK) == CLK_HCLK_APB1_Div2) || \
                           ((PCLK) == CLK_HCLK_APB1_Div4) || ((PCLK) == CLK_HCLK_APB1_Div8) || \
                           ((PCLK) == CLK_HCLK_APB1_Div16))
/**
  * @}
  */

/** @defgroup CLK_AHB2_Peripherals 
  * @{
  */  
#define CLK_AHBPeriph_RF               ((uint32_t)0x00010000)
#define CLK_AHBPeriph_SYSTICK          ((uint32_t)0x00000002)
#define CLK_AHBPeriph_DMAC             ((uint32_t)0x00000001)
#define CLK_AHBPeriph_All              ((uint32_t)0x00010003)

#define IS_CLK_AHB_PERIPH(PERIPH) ((((PERIPH) & 0x00010003) != 0x00) && ((PERIPH) != 0x00))
/**
  * @}
  */ 
  
/** @defgroup CLK_APB1_Peripherals 
  * @{
  */ 
#define CLK_APB1Periph_I2C0               ((uint32_t)0x00000001)
#define CLK_APB1Periph_SPI0               ((uint32_t)0x00000002)
#define CLK_APB1Periph_UART0              ((uint32_t)0x00000008)
#define CLK_APB1Periph_PWM01              ((uint32_t)0x00000010)
#define CLK_APB1Periph_PWM23              ((uint32_t)0x00000020)
#define CLK_APB1Periph_PWM45              ((uint32_t)0x00000040)
#define CLK_APB1Periph_PWM67              ((uint32_t)0x00000080)
#define CLK_APB1Periph_ADC                ((uint32_t)0x00000100)
#define CLK_APB1Periph_WDT                ((uint32_t)0x00000200)
#define CLK_APB1Periph_WWDT               ((uint32_t)0x00000400)
#define CLK_APB1Periph_TMR0               ((uint32_t)0x00000800)
#define CLK_APB1Periph_APB1               ((uint32_t)0x00001000)
#define CLK_APB1Periph_All                ((uint32_t)0x00000ffb)
#define IS_CLK_APB1_PERIPH(PERIPH) ((((PERIPH) & 0x00001FFB) != 0x00) && ((PERIPH) != 0x00))
/**
  * @}
  */ 
  
/** @defgroup CLK_APB2_Peripherals 
  * @{
  */ 
//#define CLK_APB2Periph_I2C1               ((uint32_t)0x00000001)
//#define CLK_APB2Periph_SPIM1              ((uint32_t)0x00000002)

#define CLK_APB2Periph_UART1              ((uint32_t)0x00000008)

#define CLK_APB2Periph_TMR1               ((uint32_t)0x00000010)
#define CLK_APB2Periph_TMR2               ((uint32_t)0x00000020)
#define CLK_APB2Periph_UART2              ((uint32_t)0x00000080)

#define CLK_APB2Periph_UART3              ((uint32_t)0x00000100)
#define CLK_APB2Periph_APB2               ((uint32_t)0x00000800)
#define CLK_APB2Periph_All                ((uint32_t)0x000001b8)
#define IS_CLK_APB2_PERIPH(PERIPH) ((((PERIPH) & 0x000009B8) != 0x00) && ((PERIPH) != 0x00))

/**
  * @}
  */ 
 
/*---------------------------------------------------------------------------------------------------------*/
/*  Module Reset Control Resister constant definitions.                                                    */
/*---------------------------------------------------------------------------------------------------------*/

#define ADC_RST  ((0x4<<24)  | CLK_IPRST1_ADCRST_Pos    ) /*!< ADC  reset is one of the CLK_ResetModule parameter */
#define PWM0_RST  ((0x4<<24) | CLK_IPRST1_PWM0RST_Pos    ) /*!< PWM  reset is one of the CLK_ResetModule parameter */
#define	WWDT_RST  ((0x4<<24) | CLK_IPRST1_WWDTRST_Pos    ) /*!< WWDT  reset is one of the CLK_ResetModule parameter */
#define	WDT_RST  ((0x4<<24)  | CLK_IPRST1_WDTRST_Pos    ) /*!< WWDT  reset is one of the CLK_ResetModule parameter */
#define UART0_RST ((0x4<<24) | CLK_IPRST1_UART0RST_Pos ) /*!< UART0 reset is one of the CLK_ResetModule parameter */
#define UART1_RST ((0x4<<24) | CLK_IPRST1_UART1RST_Pos ) /*!< UART1 reset is one of the CLK_ResetModule parameter */
#define SPI0_RST ((0x4<<24) | CLK_IPRST1_SPI0RST_Pos    ) /*!< SPI0  reset is one of the CLK_ResetModule parameter */
#define SPI1_RST ((0x4<<24) | CLK_IPRST1_SPI1RST_Pos    ) /*!< SPI1  reset is one of the CLK_ResetModule parameter */
#define I2C0_RST ((0x4<<24) | CLK_IPRST1_I2C0RST_Pos  ) /*!< I2C0  reset is one of the CLK_ResetModule parameter */
#define I2C1_RST ((0x4<<24) | CLK_IPRST1_I2C1RST_Pos  ) /*!< I2C1  reset is one of the CLK_ResetModule parameter */
#define TMR2_RST ((0x4<<24) | CLK_IPRST1_TMR2RST_Pos   ) /*!< TMR2 reset is one of the CLK_ResetModule parameter */
#define TMR1_RST ((0x4<<24) | CLK_IPRST1_TMR1RST_Pos   ) /*!< TMR1 reset is one of the CLK_ResetModule parameter */
#define TMR0_RST ((0x4<<24) | CLK_IPRST1_TMR0RST_Pos   ) /*!< TMR0 reset is one of the CLK_ResetModule parameter */
#define GPIO_RST ((0x4<<24) | CLK_IPRST1_GPIORST_Pos   ) /*!< GPIO reset is one of the CLK_ResetModule parameter */

/*---------------------------------------------------------------------------------------------------------*/
/*  Brown Out Detector Threshold Voltage Selection constant definitions.                                   */
/*---------------------------------------------------------------------------------------------------------*/
#define CLK_BODVL_3_0V             (5UL<<ANA_BOD_LVR_CTRL_BODLVL_Pos)     /*!< Setting Brown Out Detector Threshold Voltage as 3V */
#define CLK_BODVL_2_8V             (4UL<<ANA_BOD_LVR_CTRL_BODLVL_Pos)     /*!< Setting Brown Out Detector Threshold Voltage as 2.8V */

#define CLK_BODVL_2_6V             (3UL<<ANA_BOD_LVR_CTRL_BODLVL_Pos)     /*!< Setting Brown Out Detector Threshold Voltage as 2.6V */
#define CLK_BODVL_2_4V             (2UL<<ANA_BOD_LVR_CTRL_BODLVL_Pos)     /*!< Setting Brown Out Detector Threshold Voltage as 2.4V */
#define CLK_BODVL_2_2V             (1UL<<ANA_BOD_LVR_CTRL_BODLVL_Pos)     /*!< Setting Brown Out Detector Threshold Voltage as 2.2V */
#define CLK_BODVL_2_0V             (0UL<<ANA_BOD_LVR_CTRL_BODLVL_Pos)     /*!< Setting Brown Out Detector Threshold Voltage as 2V */

#define	CLK_BLDBCTL_BODDBSEL_2POW4	 0x00000001UL
#define	CLK_BLDBCTL_BODDBSEL_2POW7	 0x00000002UL
#define	CLK_BLDBCTL_BODDBSEL_2POW9	 0x00000004UL
#define	CLK_BLDBCTL_BODDBSEL_2POW11	 0x00000008UL
#define	CLK_BLDBCTL_BODDBSEL_2POW13	 0x00000010UL
#define	CLK_BLDBCTL_BODDBSEL_2POW15	 0x00000020UL

#define	CLK_BLDBCTL_LVRDBSEL_2POW4	 0x00000100UL
#define	CLK_BLDBCTL_LVRDBSEL_2POW7	 0x00000200UL
#define	CLK_BLDBCTL_LVRDBSEL_2POW9	 0x00000400UL
#define	CLK_BLDBCTL_LVRDBSEL_2POW11	 0x00000800UL
#define	CLK_BLDBCTL_LVRDBSEL_2POW13	 0x00001000UL
#define	CLK_BLDBCTL_LVRDBSEL_2POW15	 0x00002000UL
/**
  * @brief      Disable LVR detector function
  * @param      None
  * @return     None
  * @details    This macro disable LVR detector function.
  *             The register write-protection function should be disabled before using this macro.
  */
__STATIC_INLINE void CLK_DisableLvr(void)
{
    ANAC->ANA_BOD_LVR_CTRL &= ~EN_LVR_Msk;
}

/**
  * @brief      Enable LVR detector function
  * @param      None
  * @return     None
  * @details    This macro enable LVR detector function.
  *             The register write-protection function should be disabled before using this macro.
  */
__STATIC_INLINE void CLK_EnableLvr(void)
{
    ANAC->ANA_BOD_LVR_CTRL |= EN_LVR_Msk;
}

/**
  * @brief      Clear reset source flag
  * @param[in]  u32RstSrc is reset source. Including :
  *             - \ref CLK_RSTSTS_PORF_Msk
  *             - \ref CLK_RSTSTS_PINRF_Msk
  *             - \ref CLK_RSTSTS_WDTRF_Msk
  *             - \ref CLK_RSTSTS_BODRF_Msk
  *             - \ref CLK_RSTSTS_SYSRF_Msk
  *             - \ref CLK_RSTSTS_CPURF_Msk
  * @return     None
  * @details    This macro clear reset source flag.
  */
__STATIC_INLINE void CLK_ClearRstSource(uint32_t u32RstSrc)
{
    CLK->RSTSTS = u32RstSrc;
}




void CLK_ClearResetSrc(uint32_t u32Src);
uint32_t CLK_GetBODStatus(void);
uint32_t CLK_GetResetSrc(void);
uint32_t CLK_IsRegLocked(void);
uint32_t  CLK_ReadPDID(void);
void CLK_ResetChip(void);
void CLK_ResetCPU(void);
void CLK_ResetModule(uint32_t u32ModuleIndex, uint32_t u32HoldTime);
void CLK_EnableBOD(int32_t i32Mode, uint32_t u32BODLevel);
void CLK_DisableBOD(void);
void CLK_delay_10nop(uint32_t u32NopCnt);

uint32_t CLK_GetCPUFreq(void);
uint32_t CLK_GetHCLKFreq(void);
uint32_t CLK_GetPCLK1Freq(void);
uint32_t CLK_GetPCLK2Freq(void);

void CLK_SetPLLClockFreq(uint32_t dpll_sel);
uint32_t CLK_GetPLLClockFreq(void);



void CLK_RefClkSrcConfig(uint32_t u32ClkSrc);
void CLK_SYSCLKConfig(uint32_t u32ClkSrc);
void CLK_HCLKConfig(uint32_t CLK_SYSCLK);
void CLK_PCLK1Config(uint32_t u32ClkDiv );
void CLK_PCLK2Config(uint32_t u32ClkDiv);


void CLK_AHBPeriphClockCmd(uint32_t CLK_AHBPeriph, FunctionalState NewState);
void CLK_APB1PeriphClockCmd(uint32_t CLK_APB1Periph, FunctionalState NewState);
void CLK_APB2PeriphClockCmd(uint32_t CLK_APB2Periph, FunctionalState NewState);

void CLK_SysTickDelay(uint32_t us);
uint32_t CLK_WaitClockReady(uint32_t u32ClkMask);
uint32_t CLK_Wait3vSyncReady(void);

void CLK_SetWdtClkSrc(uint32_t u32clksel);
void CLK_SetWwdtClkSrc(uint32_t u32clksel);
void CLK_SetTmrClkSrc(TIMER_T *timer, uint32_t u32clksel);

void CLK_AdcReset(void);

void CLK_EnableTempDetect(void);
void CLK_DisableTempDetect(void);

void SYS_ResetModule(uint32_t u32ModuleIndex, uint32_t u32HoldTime);



#ifdef __cplusplus
}
#endif

#endif //__CLK_H


