/*******************************************************************************
 * @note Copyright (C) 2019 Shanghai Panchip Microelectronics Co., Ltd.
 * All rights reserved.
 *
 * @file 'lowpower.h'
 * @brief 
 *
 * @history - V0.01, 2019-03-20, yangyaming, first implementation.
*******************************************************************************/
#ifndef __LOWPOWER_H
#define __LOWPOWER_H

#ifdef __cplusplus
extern "C"{
#endif //__cplusplus

#include "pan2025xx.h"


////usr macro,not register
#define LP_WKUP_MODE_SEL_GPIO                           (0)
#define LP_WKUP_MODE_SEL_32K                            (1)

#define LP_WKUP_SRAM_DATA_SAVED                         (0)
#define LP_WKUP_SRAM_DATA_LOSTED                        (1)
/**
* @brief  This function control the sleep function
* @param[in]  anac: where anac is anac module base address
* @return   none
*/
__STATIC_INLINE void LP_EnSleepFunc(ANAC_T *anac)
{
    anac->LP_FL_CTRL0 |= SLEEP_EN_Msk;
}

/**
* @brief  This function control the sleep function
* @param[in]  anac: where anac is anac module base address
* @param[in]  IntSel: interrupt enable select,it could be:
*            LP_SLEEP_INT_EN_Msk    deepsleep and sleep wake up interrupt enable bit	
*            LP_STANDBY_INT_EN_Msk
* @return   none
*/
__STATIC_INLINE void LP_InterruptEnable(ANAC_T *anac,uint32_t IntSel)
{
    anac->LP_INT_CTRL |= IntSel;
}

void LP_RequestSleep(ANAC_T *anac);
void LP_SetSleepTime(ANAC_T *anac,uint32_t u32ClkCnt);
void LP_ClearWakeFlag(ANAC_T *anac,uint32_t u32Mask);
void LP_SetExternalWake(ANAC_T *anac,uint8_t u8ExtWkDis);
void LP_SetDigitalDelay(ANAC_T *anac,uint8_t u8Clk32Cnt);
void LP_DisableDigitalRst(ANAC_T *anac);	
void LP_EnableLpLdo(ANAC_T *anac);

void LP_SetDeepSleepConfig(ANAC_T *anac);
void LP_SetStandbyConfig(ANAC_T *anac,uint8_t u8SramRetention);

#ifdef __cplusplus
}
#endif //__cplusplus

#endif //__LOWPOWER_H
