/*******************************************************************************
 * @note      Copyright (C) 2019 Shanghai Panchip Microelectronics Co., Ltd.
 *            All rights reserved.
 * @file      sys.h
 * @brief     PAN2025 series SYS driver header file.
 * @history - V1.0, 2019-3-8, zhangzhao, first implementation.
*******************************************************************************/
#ifndef __SYS_H
#define __SYS_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "pan2025xx.h"    

/*---------------------------------------------------------------------------------------------------------*/
/*  Multi-Function constant definitions.                                                                   */
/*---------------------------------------------------------------------------------------------------------*/
#define SYS_MFP_TYPE_Msk(bit)       (1UL << ((bit) +16)) /*!< TYPE mask for Multiple Function Port */
#define SYS_MFP_ALT_Msk(bit)        (1UL << ((bit) + 8)) /*!< ALT mask for Multiple Function Port */
#define SYS_MFP_MFP_Msk(bit)        (1UL << ((bit)    )) /*!< MFP mask for Multiple Function Port */
                                                                                         
#define SYS_MFP_P00_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P00_TADC_CLK         0x00000001UL
#define SYS_MFP_P00_PWM0_CH2       	 0x00000100UL     /*!< PWM0 channel 2 output/capture input.             */
#define SYS_MFP_P00_UART0_TXD        0x00000101UL     /*!< Data transmitter output pin for UART0.           */
#define SYS_MFP_P00_SPI0_SS          0x00010100UL     /*!< 1st SPI0 slave select pin.                       */
#define SYS_MFP_P00_PWM0_CH5         0x00010101UL     /*!< PWM0 channel 5 output/capture input.             */
#define SYS_MFP_P00_Msk              0x00010101UL     /*!< P0_MFP pin 0 mask.                               */
                                                       
#define SYS_MFP_P01_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P01_SPI0_SS          0x00000002UL     /*!< 1st SPI0 slave select pin.                       */
#define SYS_MFP_P01_PWM0_CH3       	 0x00000200UL     /*!< PWM0 channel 3 output/capture input.             */
#define SYS_MFP_P01_UART0_RXD        0x00000202UL     /*!< Data receiver input pin for UART0.               */
#define SYS_MFP_P01_ADC_CH6          0x00020002UL     /*!< ADC channel 6 analog input.                      */
#define SYS_MFP_P01_Msk              0x00020202UL     /*!< P0_MFP pin 1 mask.                               */
                                                        
#define SYS_MFP_P02_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */ 
#define SYS_MFP_P02_Msk              0x00040404UL     /*!< P0_MFP pin 2 mask.                               */

#define SYS_MFP_P03_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */ 
#define SYS_MFP_P03_SPI1_SS          0x00000008UL     /*!< 1st SPI1 slave select pin.                       */
#define SYS_MFP_P03_Msk              0x00080808UL     /*!< P0_MFP pin 3 mask.                               */

#define SYS_MFP_P04_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P04_TADC_DATH      	 0x00000010UL
#define SYS_MFP_P04_SPI0_SS          0x00001000UL     /*!< 1st SPI0 slave select pin.                       */
#define SYS_MFP_P04_PWM0_CH5         0x00001010UL     /*!< PWM0 channel 5 output/capture input.             */
#define SYS_MFP_P04_TM1_EXT          0x00100010UL     /*!< Timer1 external capture input.                   */
#define SYS_MFP_P04_Msk              0x00101010UL     /*!< P0_MFP pin 4 mask.                               */
                                                        
#define SYS_MFP_P05_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P05_UART3_TXD        0x00000020UL     /*!< Data transmitter output pin for UART3.           */
#define SYS_MFP_P05_SPI0_MOSI        0x00002000UL     /*!< 1st SPI0 MOSI (Master Out, Slave In) pin.        */
#define SYS_MFP_P05_PWM0_CH4         0x00002020UL     /*!< PWM0 channel 4 output/capture input.             */
#define SYS_MFP_P05_Msk              0x00202020UL     /*!< P0_MFP pin 5 mask.                               */
                                                        
#define SYS_MFP_P06_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P06_UART3_RXD        0x00000040UL     /*!< Data receiver input pin for UART3.               */
#define SYS_MFP_P06_SPI0_MISO        0x00004000UL     /*!< 1st SPI0 MISO (Master In, Slave Out) pin.        */
#define SYS_MFP_P06_PWM0_CH1         0x00004040UL     /*!< PWM0 channel 1 output/capture input.             */
#define SYS_MFP_P06_Msk              0x00404040UL     /*!< P0_MFP pin 6 mask.                               */
                                                        
#define SYS_MFP_P07_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P07_SPI0_CLK         0x00008000UL     /*!< SPI0 serial clock pin.                           */
#define SYS_MFP_P07_PWM0_CH0         0x00008080UL     /*!< PWM0 channel 0 output/capture input.             */
#define SYS_MFP_P07_Msk              0x00808080UL     /*!< P0_MFP pin 7 mask.                               */

#define SYS_MFP_P10_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P10_ADC_CH1          0x00000001UL     /*!< ADC channel 1 analog input.                      */
#define SYS_MFP_P10_SPI0_SS          0x00000101UL     /*!< 1st SPI0 slave select pin.                       */
#define SYS_MFP_P10_Msk              0x00010101UL     /*!< P1_MFP pin 0 mask.                               */

#define SYS_MFP_P11_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */ 
#define SYS_MFP_P11_SPI1_CLK         0x00000002UL     /*!< SPI0 serial clock pin.                           */ 
#define SYS_MFP_P11_UART2_TXD        0x00000200UL     /*!< Data transmitter output pin for UART2.           */
#define SYS_MFP_P11_UART4_TXD        0x00000202UL     /*!< Data transmitter output pin for UART4.           */
#define SYS_MFP_P11_Msk              0x00020202UL     /*!< P1_MFP pin 1 mask.                               */ 

#define SYS_MFP_P12_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P12_ADC_CH2          0x00000004UL     /*!< ADC channel 2 analog input.                      */
#define SYS_MFP_P12_UART0_RXD        0x00000400UL     /*!< Data receiver input pin for UART0.               */
#define SYS_MFP_P12_PWM0_CH0         0x00000404UL     /*!< PWM0 channel 0 output/capture input.             */
#define SYS_MFP_P12_SPI0_MISO        0x00040004UL     /*!< 1st SPI0 MISO (Master In, Slave Out) pin.        */
#define SYS_MFP_P12_Msk              0x00040404UL     /*!< P1_MFP pin 2 mask.                               */

#define SYS_MFP_P13_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P13_ADC_CH3          0x00000008UL     /*!< ADC channel 3 analog input.                      */
#define SYS_MFP_P13_UART0_TXD        0x00000800UL     /*!< Data transmitter output pin for UART0.           */
#define SYS_MFP_P13_PWM0_CH1         0x00000808UL     /*!< PWM0 channel 1 output/capture input.             */
#define SYS_MFP_P13_SPI0_MOSI        0x00080008UL     /*!< 1st SPI0 MOSI (Master Out, Slave In) pin.        */
#define SYS_MFP_P13_Msk              0x00080808UL     /*!< P1_MFP pin 3 mask.                               */

#define SYS_MFP_P14_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P14_ADC_CH4          0x00000010UL     /*!< ADC channel 4 analog input.                      */
#define SYS_MFP_P14_UART1_RXD        0x00001000UL     /*!< Data receiver input pin for UART1.               */
#define SYS_MFP_P14_PWM0_CH4         0x00001010UL     /*!< PWM0 channel 4 output/capture input.             */
#define SYS_MFP_P14_SPI0_CLK         0x00100010UL     /*!< SPI0 serial clock pin.                           */
#define SYS_MFP_P14_Msk              0x00101010UL     /*!< P1_MFP pin 4 mask.                               */
    
#define SYS_MFP_P15_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */    
#define SYS_MFP_P15_ADC_CH5          0x00000020UL     /*!< ADC channel 5 analog input.                      */
#define SYS_MFP_P15_UART1_TXD        0x00002000UL     /*!< Data transmitter output pin for UART1.           */
#define SYS_MFP_P15_32K_RCO          0x00002020UL     /*!< 32K RCO output                    pin.           */
#define SYS_MFP_P15_SPI0_SS          0x00200020UL     /*!< 1st SPI0 slave select pin.                       */
#define SYS_MFP_P15_Msk              0x00202020UL     /*!< P1_MFP pin 5 mask.                               */

#define SYS_MFP_P16_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */ 
#define SYS_MFP_P16_SPI1_MOSI        0x00000040UL     /*!< 1st SPI1 MOSI (Master Out, Slave In) pin.        */
#define SYS_MFP_P16_UART2_RXD        0x00004000UL     /*!< Data receiver input pin for UART2.               */
#define SYS_MFP_P16_UART4_RXD        0x00004040UL     /*!< Data receiver input pin for UART4.               */
#define SYS_MFP_P16_Msk              0x00404040UL     /*!< P1_MFP pin 6 mask.                               */ 
 
#define SYS_MFP_P17_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */  
#define SYS_MFP_P17_SPI1_MISO        0x00000080UL     /*!< 1st SPI1 MISO (Master In, Slave Out) pin.        */ 
#define SYS_MFP_P17_UART4_CTS        0x00008080UL     /*!< UART CTS pin for UART4.                          */
#define SYS_MFP_P17_Msk              0x00808080UL     /*!< P1_MFP pin 7 mask.                               */ 

#define SYS_MFP_P20_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */ 
#define SYS_MFP_P20_SPI1_SS          0x00000001UL     /*!< 1st SPI1 slave select pin.                       */
#define SYS_MFP_P20_INT2             0x00000100UL     /*!< External interrupt2 input pin.                   */
#define SYS_MFP_P20_UART4_RTS        0x00000101UL     /*!< UART RTS pin for UART4.                          */
#define SYS_MFP_P20_Msk              0x00010101UL     /*!< P2_MFP pin 0 mask.                               */ 
                                                        
#define SYS_MFP_P21_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */  
#define SYS_MFP_P21_Msk              0x00020202UL     /*!< P2_MFP pin 1 mask.                               */ 
                                                        
#define SYS_MFP_P22_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P22_UART2_TXD        0x00000004UL     /*!< Data transmitter output pin for UART2.           */
#define SYS_MFP_P22_PWM0_CH0         0x00000400UL     /*!< PWM0 channel 0 output/capture input.             */
#define SYS_MFP_P22_I2C1_SCL         0x00000404UL     /*!< I2C1 clock pin.                                  */
#define SYS_MFP_P22_I2C0_SCL         0x00040004UL     /*!< I2C0 clock pin.                                  */
#define SYS_MFP_P22_Msk              0x00040404UL     /*!< P2_MFP pin 2 mask.                               */
                                                     
#define SYS_MFP_P23_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P23_UART2_RXD        0x00000008UL     /*!< Data receiver input pin for UART2.               */
#define SYS_MFP_P23_PWM0_CH1         0x00000800UL     /*!< PWM0 channel 1 output/capture input.             */
#define SYS_MFP_P23_I2C1_SDA         0x00000808UL     /*!< I2C1 data input/output pin.                      */
#define SYS_MFP_P23_I2C0_SDA         0x00080008UL     /*!< I2C0 data input/output pin.                      */
#define SYS_MFP_P23_Msk              0x00080808UL     /*!< P2_MFP pin 3 mask.                               */
                                                        
#define SYS_MFP_P24_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P24_UART1_RXD        0x00000010UL     /*!< Data receiver input pin for UART1.               */
#define SYS_MFP_P24_PWM0_CH2         0x00001000UL     /*!< PWM0 channel 2 output/capture input.             */
#define SYS_MFP_P24_16M_RCO          0x00001010UL
#define SYS_MFP_P24_Msk              0x00101010UL     /*!< P2_MFP pin 4 mask.                               */
                                                        
#define SYS_MFP_P25_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P25_UART1_TXD        0x00000020UL     /*!< Data transmitter output pin for UART1.           */
#define SYS_MFP_P25_PWM0_CH3         0x00002000UL     /*!< PWM0 channel 3 output/capture input.             */
#define SYS_MFP_P25_Msk              0x00202020UL     /*!< P2_MFP pin 5 mask.                               */

#define SYS_MFP_P26_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P26_SPI0_SS          0x00000040UL     /*!< 1st SPI0 slave select pin.                       */ 
#define SYS_MFP_P26_PWM0_CH4         0x00004000UL     /*!< PWM0 channel 4 output/capture input.             */
#define SYS_MFP_P26_Msk              0x00404040UL     /*!< P2_MFP pin 6 mask.                               */
                                                        
#define SYS_MFP_P27_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */ 
#define SYS_MFP_P27_TM2_EXT          0x00008000UL     /*!< Timer2 external capture input.                   */ 
#define SYS_MFP_P27_TADC_VLD         0x00008080UL
#define SYS_MFP_P27_Msk              0x00808080UL     /*!< P2_MFP pin 7 mask.                               */ 
                                                        
#define SYS_MFP_P30_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P30_Msk              0x00010101UL     /*!< P3_MFP pin 0 mask.                               */
                                                        
#define SYS_MFP_P31_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */                                                        
#define SYS_MFP_P31_ADC_CH7          0x00000202UL     /*!< ADC channel 7 analog input.                      */
#define SYS_MFP_P31_Msk              0x00020202UL     /*!< P3_MFP pin 1 mask.                               */
                                                        
#define SYS_MFP_P32_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P32_INT0             0x00000004UL     /*!< External interrupt0 input pin.                   */
#define SYS_MFP_P32_TM0_EXT          0x00000400UL     /*!< Timer0 external capture input.                   */
#define SYS_MFP_P32_STADC            0x00000404UL     /*!< ADC external trigger input.                      */
#define SYS_MFP_P32_Msk              0x00040404UL     /*!< P3_MFP pin 2 mask.                               */
                                                        
#define SYS_MFP_P34_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P34_TM0_CNT_OUT      0x00000010UL     /*!< Timer0 event counter input/toggle output.        */
#define SYS_MFP_P34_I2C0_SDA         0x00001000UL     /*!< I2C0 data input/output pin.                      */
#define SYS_MFP_P34_UART3_TXD        0x00001010UL     /*!< Data transmitter output pin for UART3.           */
#define SYS_MFP_P34_Msk              0x00101010UL     /*!< P3_MFP pin 4 mask.                               */
                                                        
#define SYS_MFP_P35_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P35_TM1_CNT_OUT      0x00000020UL     /*!< Timer1  event counter input/toggle output.       */
#define SYS_MFP_P35_I2C0_SCL         0x00002000UL     /*!< I2C0 clock pin.                                  */
#define SYS_MFP_P35_UART3_RXD        0x00002020UL     /*!< Data receiver input pin for UART3.               */
#define SYS_MFP_P35_Msk              0x00202020UL     /*!< P3_MFP pin 5 mask.                               */

#define SYS_MFP_P36_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P36_TM1_EXT          0x00000040UL     /*!< Timer1 external capture input.                   */
#define SYS_MFP_P36_CLKO             0x00004000UL     /*!< Clock output.                                    */
#define SYS_MFP_P36_Msk              0x00404040UL     /*!< P3_MFP pin 6 mask.                               */
                                                        
#define SYS_MFP_P46_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P46_ICE_CLK          0x00000040UL     /*!< Serial wired debugger clock pin.                 */
#define SYS_MFP_P46_UART1_RXD        0x00004000UL     /*!< Data receiver input pin for UART1.               */
#define SYS_MFP_P46_I2C0_SCL         0x00004040UL     /*!< clk pin for IIC0.                                */
#define SYS_MFP_P46_Msk              0x00404040UL     /*!< P4_MFP pin 6 mask.                               */
                                                        
#define SYS_MFP_P47_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P47_ICE_DAT          0x00000080UL     /*!< Serial wired debugger data pin.                  */
#define SYS_MFP_P47_UART1_TXD        0x00008000UL     /*!< Data transmitter output pin for UART1.           */
#define SYS_MFP_P47_I2C0_SDA         0x00008080UL     /*!< Data receiver input pin for IIC0.                */
#define SYS_MFP_P47_Msk              0x00808080UL     /*!< P4_MFP pin 7 mask.                               */
                                                        
#define SYS_MFP_P50_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P50_PWM0_CH7         0x00000001UL     /*!< PWM0 channel 7 output/capture input.             */
#define SYS_MFP_P50_I2C1_SDA         0x00000100UL     /*!< I2C1 data input/output pin.                      */
#define SYS_MFP_P50_UART0_TXD        0x00000101UL     /*!< Data transmitter output pin for UART0.           */
#define SYS_MFP_P50_I2C0_SDA         0x00010001UL     /*!< I2C0 SDA pin.                                    */
#define SYS_MFP_P50_Msk              0x00010101UL     /*!< P5_MFP pin 0 mask.                               */ 
                                                        
#define SYS_MFP_P51_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P51_PWM0_CH6         0x00000002UL     /*!< PWM0 channel 6 output/capture input.             */
#define SYS_MFP_P51_I2C1_SCL         0x00000200UL     /*!< I2C1 clock pin.                                  */
#define SYS_MFP_P51_UART0_RXD        0x00000202UL     /*!< Data receiver input pin for UART0.               */
#define SYS_MFP_P51_I2C0_SCL         0x00020002UL     /*!< I2C0 clock pin.                                  */
#define SYS_MFP_P51_Msk              0x00020202UL     /*!< P5_MFP pin 1 mask.                               */ 
                                                        
#define SYS_MFP_P52_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P52_INT1             0x00000004UL     /*!< External interrupt1 input pin.                   */
#define SYS_MFP_P52_Msk              0x00040404UL     /*!< P5_MFP pin 2 mask.                               */
                                                        
#define SYS_MFP_P53_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */                                                        
#define SYS_MFP_P53_ADC_CH0          0x00000008UL     /*!< ADC channel 0 analog input.                      */
#define SYS_MFP_P53_SPI0_MISO        0x00000808UL     /*!< 1st SPI0 MISO (Master In, Slave Out) pin.        */
#define SYS_MFP_P53_Msk              0x00080808UL     /*!< P5_MFP pin 3 mask.                               */

#define SYS_MFP_P54_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */
#define SYS_MFP_P54_TM0_CNT_OUT      0x00000010UL     /*!< Timer0 event counter input/toggle output.        */ 
#define SYS_MFP_P54_TDAC_DATL        0x00001000UL
#define SYS_MFP_P54_SPI0_MOSI        0x00001010UL     /*!< 1st SPI0 MOSI (Master Out, Slave In) pin.        */
#define SYS_MFP_P54_Msk              0x00101010UL     /*!< P5_MFP pin 4 mask.                               */
                                                        
#define SYS_MFP_P55_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */ 
#define SYS_MFP_P55_SPI0_CLK         0x00000020UL     /*!< SPI0 serial clock pin.                           */
#define SYS_MFP_P55_SPI0_SS          0x00002000UL     /*!< 1st SPI0 slave select pin.                       */
#define SYS_MFP_P55_PWM0_CH5         0x00002020UL     /*!< PWM0 channel 5 output/capture input.             */ 
#define SYS_MFP_P55_TADC_CLK         0x00200020UL
#define SYS_MFP_P55_PWM0_CH2         0x00202000UL     /*!< PWM0 channel 2 output/capture input.             */ 
#define SYS_MFP_P55_UART0_TXD        0x00202020UL     /*!< Data transmitter output pin for UART0.           */ 
#define SYS_MFP_P55_Msk              0x00202020UL     /*!< P5_MFP pin 5 mask.                               */

#define SYS_MFP_P56_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */ 
#define SYS_MFP_P56_UART4_TXD        0x00000040UL     /*!< Data transmitter output pin for UART4.           */
#define SYS_MFP_P56_I2C0_SCL         0x00004000UL     /*!< I2C0 clock pin.                                  */
#define SYS_MFP_P56_PWM0_CH6         0x00004040UL     /*!< PWM0 channel 6 output/capture input.             */ 
#define SYS_MFP_P56_I2C1_SCL         0x00404000UL     /*!< I2C1 clock pin.                                  */
#define SYS_MFP_P56_UART0_RXD        0x00404040UL     /*!< Data receiver input pin for UART0.               */
#define SYS_MFP_P56_Msk              0x00404040UL     /*!< P5_MFP pin 6 mask.                               */ 

#define SYS_MFP_P57_GPIO             0x00000000UL     /*!< General purpose digital I/O pin.                 */ 
#define SYS_MFP_P56_UART4_RXD        0x00000080UL     /*!< Data receiver input pin for UART4.               */
#define SYS_MFP_P57_I2C0_SDA         0x00008000UL     /*!< I2C0 data input/output pin.                      */
#define SYS_MFP_P57_PWM0_CH7         0x00008080UL     /*!< PWM0 channel 7 output/capture input.             */ 
#define SYS_MFP_P57_I2C1_SDA         0x00808000UL     /*!< I2C1 data input/output pin.                      */
#define SYS_MFP_P57_UART0_TXD        0x00808080UL     /*!< Data transmitter output pin for UART0.           */
#define SYS_MFP_P57_Msk              0x00808080UL     /*!< P5_MFP pin 6 mask.                               */ 


#define	SYS_BLDBCTL_BODDBSEL_2POW4	 0x00000001UL
#define	SYS_BLDBCTL_BODDBSEL_2POW7	 0x00000002UL
#define	SYS_BLDBCTL_BODDBSEL_2POW9	 0x00000004UL
#define	SYS_BLDBCTL_BODDBSEL_2POW11	 0x00000008UL
#define	SYS_BLDBCTL_BODDBSEL_2POW13	 0x00000010UL
#define	SYS_BLDBCTL_BODDBSEL_2POW15	 0x00000020UL

#define	SYS_BLDBCTL_LVRDBSEL_2POW4	 0x00000100UL
#define	SYS_BLDBCTL_LVRDBSEL_2POW7	 0x00000200UL
#define	SYS_BLDBCTL_LVRDBSEL_2POW9	 0x00000400UL
#define	SYS_BLDBCTL_LVRDBSEL_2POW11	 0x00000800UL
#define	SYS_BLDBCTL_LVRDBSEL_2POW13	 0x00001000UL
#define	SYS_BLDBCTL_LVRDBSEL_2POW15	 0x00002000UL

/*@}*/ /* end of group PAN2025_SYS_EXPORTED_CONSTANTS */

/** @addtogroup PAN2025_SYS_EXPORTED_FUNCTIONS SYS Exported Functions
  @{
*/
/**
  * @brief      Clear Brown-out detector interrupt flag
  * @param      None
  * @return     None
  * @details    This macro clear Brown-out detector interrupt flag.
  */
#define SYS_CLEAR_BOD_INT_FLAG()        (SYS->BODCTL |= SYS_BODCTL_BODIF_Msk)

/**
  * @brief      Set Brown-out detector function to normal mode
  * @param      None
  * @return     None
  * @details    This macro set Brown-out detector to normal mode.
  *             The register write-protection function should be disabled before using this macro.
  */
#define SYS_CLEAR_BOD_LPM()             (SYS->BODCTL &= ~SYS_BODCTL_BODLPM_Msk)

/**
  * @brief      Disable Brown-out detector function
  * @param      None
  * @return     None
  * @details    This macro disable Brown-out detector function.
  *             The register write-protection function should be disabled before using this macro.
  */
#define SYS_DISABLE_BOD()               (SYS->BODCTL = (SYS->BODCTL &~(SYS_BODCTL_BODVL_Msk|SYS_BODCTL_BODEN_Msk))|SYS_BODCTL_BODVL_Msk)

/**
  * @brief      Enable Brown-out detector function
  * @param      None
  * @return     None
  * @details    This macro enable Brown-out detector function.
  *             The register write-protection function should be disabled before using this macro.
  */
#define SYS_ENABLE_BOD()                (SYS->BODCTL = (SYS->BODCTL | SYS_BODCTL_BODEN_Msk))

/**
  * @brief      Get Brown-out detector interrupt flag
  * @param      None
  * @retval     0   Brown-out detect interrupt flag is not set.
  * @retval     >=1 Brown-out detect interrupt flag is set.
  * @details    This macro get Brown-out detector interrupt flag.
  */
#define SYS_GET_BOD_INT_FLAG()          (SYS->BODCTL & SYS_BODCTL_BODIF_Msk)

/**
  * @brief      Get Brown-out detector status
  * @param      None
  * @retval     0   System voltage is higher than BOD threshold voltage setting or BOD function is disabled.
  * @retval     >=1 System voltage is lower than BOD threshold voltage setting.
  * @details    This macro get Brown-out detector output status.
  *             If the BOD function is disabled, this function always return 0.
  */
#define SYS_GET_BOD_OUTPUT()            (SYS->BODCTL & SYS_BODCTL_BODOUT_Msk)

/**
  * @brief      Enable Brown-out detector interrupt function
  * @param      None
  * @return     None
  * @details    This macro enable Brown-out detector interrupt function.
  *             The register write-protection function should be disabled before using this macro.
  */
#define SYS_DISABLE_BOD_RST()           (SYS->BODCTL &= ~SYS_BODCTL_BODRSTEN_Msk)

/**
  * @brief      Enable Brown-out detector reset function
  * @param      None
  * @return     None
  * @details    This macro enable Brown-out detect reset function.
  *             The register write-protection function should be disabled before using this macro.
  */
#define SYS_ENABLE_BOD_RST()            (SYS->BODCTL |= SYS_BODCTL_BODRSTEN_Msk)

/**
  * @brief      Set Brown-out detector function low power mode
  * @param      None
  * @return     None
  * @details    This macro set Brown-out detector to low power mode.
  *             The register write-protection function should be disabled before using this macro.
  */
#define SYS_SET_BOD_LPM()               (SYS->BODCTL |= SYS_BODCTL_BODLPM_Msk)

/**
  * @brief      Set Brown-out detector voltage level
  * @param[in]  u32Level is Brown-out voltage level. Including :
  *             - \ref SYS_BODCTL_BODVL_4_4V
  *             - \ref SYS_BODCTL_BODVL_3_7V
  *             - \ref SYS_BODCTL_BODVL_2_7V
  *             - \ref SYS_BODCTL_BODVL_2_2V
  * @return     None
  * @details    This macro set Brown-out detector voltage level.
  *             The write-protection function should be disabled before using this macro.
  */
#define SYS_SET_BOD_LEVEL(u32Level)     (SYS->BODCTL = (SYS->BODCTL & ~SYS_BODCTL_BODVL_Msk) | (u32Level))

/**
  * @brief      Get reset source is from Brown-out detector reset
  * @param      None
  * @retval     0   Previous reset source is not from Brown-out detector reset
  * @retval     >=1 Previous reset source is from Brown-out detector reset
  * @details    This macro get previous reset source is from Brown-out detect reset or not.
  */
#define SYS_IS_BOD_RST()                (SYS->RSTSTS & SYS_RSTSTS_BODRF_Msk)

/**
  * @brief      Set BOD De-Bounce (glitch) time Control register
  * @param      Mask bit of select.[5:0]
  *				ref:/ SYS_BLDBCTL_BODDBSEL_2POW4 ...
  * @details    This macro get previous reset source is from Brown-out detect reset or not.
  */
#define	SYS_SET_BODDBSEL(u32Msk)		(SYS->BLDBCTL = ((SYS->BLDBCTL & ~SYS_BLDBCTL_BODDBSEL_Msk) | (u32Msk )))

/**
  * @brief      Set LVR De-Bounce (glitch) time Control register
  * @param      Mask bit of select.[13:8]
  *				ref:/ SYS_BLDBCTL_LVRDBSEL_2POW4 ...
  * @details    This macro get previous reset source is from Brown-out detect reset or not.
  */
#define	SYS_SET_LVRDBSEL(u32Msk)		(SYS->BLDBCTL = ((SYS->BLDBCTL & ~SYS_BLDBCTL_LVRDBSEL_Msk) | (u32Msk )))

/**
  * @brief      Get reset source is from CPU reset
  * @param      None
  * @retval     0   Previous reset source is not from CPU reset
  * @retval     >=1 Previous reset source is from CPU reset
  * @details    This macro get previous reset source is from CPU reset.
  */
#define SYS_IS_CPU_RST()                (SYS->RSTSTS & SYS_RSTSTS_CPURF_Msk)

/**
  * @brief      Get reset source is from Power-on Reset
  * @param      None
  * @retval     0   Previous reset source is not from Power-on Reset
  * @retval     >=1 Previous reset source is from Power-on Reset
  * @details    This macro get previous reset source is from Power-on Reset.
  */
#define SYS_IS_POR_RST()                (SYS->RSTSTS & SYS_RSTSTS_PORF_Msk)

/**
  * @brief      Get reset source is from reset pin reset
  * @param      None
  * @retval     0   Previous reset source is not from reset pin reset
  * @retval     >=1 Previous reset source is from reset pin reset
  * @details    This macro get previous reset source is from reset pin reset.
  */
#define SYS_IS_RSTPIN_RST()             (SYS->RSTSTS & SYS_RSTSTS_PINRF_Msk)

/**
  * @brief      Get reset source is from system reset
  * @param      None
  * @retval     0   Previous reset source is not from system reset
  * @retval     >=1 Previous reset source is from system reset
  * @details    This macro get previous reset source is from system reset.
  */
#define SYS_IS_SYSTEM_RST()             (SYS->RSTSTS & SYS_RSTSTS_SYSRF_Msk)

/**
  * @brief      Get reset source is from window watch dog reset
  * @param      None
  * @retval     0   Previous reset source is not from window watch dog reset
  * @retval     >=1 Previous reset source is from window watch dog reset
  * @details    This macro get previous reset source is from window watch dog reset.
  */
#define SYS_IS_WDT_RST()                (SYS->RSTSTS & SYS_RSTSTS_WDTRF_Msk)

/**
  * @brief      Disable Power-on Reset function
  * @param      None
  * @return     None
  * @details    This macro disable Power-on Reset function.
  *             The register write-protection function should be disabled before using this macro.
  */
#define SYS_DISABLE_POR()               (SYS->PORCTL = 0x5AA5)

/**
  * @brief      Enable Power-on Reset function
  * @param      None
  * @return     None
  * @details    This macro enable Power-on Reset function.
  *             The register write-protection function should be disabled before using this macro.
  */
#define SYS_ENABLE_POR()                (SYS->PORCTL = 0)

/**
  * @brief      Clear reset source flag
  * @param[in]  u32RstSrc is reset source. Including :
  *             - \ref SYS_RSTSTS_PORF_Msk
  *             - \ref SYS_RSTSTS_PINRF_Msk
  *             - \ref SYS_RSTSTS_WDTRF_Msk
  *             - \ref SYS_RSTSTS_BODRF_Msk
  *             - \ref SYS_RSTSTS_SYSRF_Msk
  *             - \ref SYS_RSTSTS_CPURF_Msk
  * @return     None
  * @details    This macro clear reset source flag.
  */
#define SYS_CLEAR_RST_SOURCE(u32RstSrc) ((SYS->RSTSTS) = (u32RstSrc) )


/**
  * @brief      Disable register write-protection function
  * @param      None
  * @return     None
  * @details    This function disable register write-protection function.
  *             To unlock the protected register to allow write access.
  */
__STATIC_INLINE void SYS_UnlockReg(void)
{
    do
     {
        SYS->REGLCTL = 0x59;
        SYS->REGLCTL = 0x16;
        SYS->REGLCTL = 0x88;
    }
    while(SYS->REGLCTL == 0);
}

/**
  * @brief      Enable register write-protection function
  * @param      None
  * @return     None
  * @details    This function is used to enable register write-protection function.
  *             To lock the protected register to forbid write access.
  */
__STATIC_INLINE void SYS_LockReg(void)
{
    SYS->REGLCTL = 0;
}




void SYS_delay_10nop(uint32_t u32NopCnt);


#ifdef __cplusplus
}
#endif

#endif //__SYS_H__
