/*******************************************************************************
 * @note Copyright (C) 2019 Shanghai Panchip Microelectronics Co., Ltd.
 * All rights reserved.
 *
 * @file wdt.h
 * @brief 
 *
 * @history - V0.01, 2019-03-20, yangyaming, first implementation.
*******************************************************************************/

#ifndef __WDT_H
#define __WDT_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "pan2025xx.h" 


typedef enum _WDT_Timeout
{
    WDT_TIMEOUT_2POW4   = (0UL << WDT_CTL_TOUTSEL_Pos), /*!< WDT setting for timeout interval = 2^4 * WDT clocks */
    WDT_TIMEOUT_2POW6   = (1UL << WDT_CTL_TOUTSEL_Pos), /*!< WDT setting for timeout interval = 2^6 * WDT clocks */
    WDT_TIMEOUT_2POW8   = (2UL << WDT_CTL_TOUTSEL_Pos), /*!< WDT setting for timeout interval = 2^8 * WDT clocks */
    WDT_TIMEOUT_2POW10  = (3UL << WDT_CTL_TOUTSEL_Pos), /*!< WDT setting for timeout interval = 2^10 * WDT clocks */
    WDT_TIMEOUT_2POW12  = (4UL << WDT_CTL_TOUTSEL_Pos), /*!< WDT setting for timeout interval = 2^12 * WDT clocks */
    WDT_TIMEOUT_2POW14  = (5UL << WDT_CTL_TOUTSEL_Pos), /*!< WDT setting for timeout interval = 2^14 * WDT clocks */
    WDT_TIMEOUT_2POW16  = (6UL << WDT_CTL_TOUTSEL_Pos), /*!< WDT setting for timeout interval = 2^16 * WDT clocks */
    WDT_TIMEOUT_2POW18  = (7UL << WDT_CTL_TOUTSEL_Pos)  /*!< WDT setting for timeout interval = 2^18 * WDT clocks */
} WDT_TimeoutDef;

typedef enum _WDT_ResetDelay
{
    WDT_RESET_DELAY_2CLK    = (3UL << WDT_ALTCTL_RSTDSEL_Pos),  /*!< WDT setting reset delay to 2 WDT clocks */
    WDT_RESET_DELAY_17CLK   = (2UL << WDT_ALTCTL_RSTDSEL_Pos),  /*!< WDT setting reset delay to 17 WDT clocks */
    WDT_RESET_DELAY_129CLK  = (1UL << WDT_ALTCTL_RSTDSEL_Pos),  /*!< WDT setting reset delay to 129 WDT clocks */
    WDT_RESET_DELAY_1025CLK = (0UL << WDT_ALTCTL_RSTDSEL_Pos)   /*!< WDT setting reset delay to 1025 WDT clocks */
} WDT_ResetDelayDef;

/** 
  * @brief This macro clear WDT time-out reset system flag. 
  * \hideinitializer
  */
#define WDT_CLEAR_RESET_FLAG()  (WDT->CTL = (WDT->CTL & ~(WDT_CTL_IF_Msk | WDT_CTL_WKF_Msk | WDT_CTL_TOF_Msk)) | WDT_CTL_RSTF_Msk) 

/** 
  * @brief This macro clear WDT time-out flag. 
  * \hideinitializer
  */
#define WDT_CLEAR_TIMEOUT_FLAG() (WDT->CTL = (WDT->CTL & ~(WDT_CTL_RSTF_Msk | WDT_CTL_WKF_Msk | WDT_CTL_IF_Msk)) | WDT_CTL_TOF_Msk)


/** 
  * @brief This macro clear WDT time-out interrupt flag. 
  * \hideinitializer
  */
#define WDT_CLEAR_TIMEOUT_INT_FLAG() (WDT->CTL = (WDT->CTL & ~(WDT_CTL_RSTF_Msk | WDT_CTL_WKF_Msk | WDT_CTL_TOF_Msk)) | WDT_CTL_IF_Msk)

/** 
  * @brief This macro clear WDT time-out wake-up system flag. 
  * \hideinitializer
  */
#define WDT_CLEAR_TIMEOUT_WAKEUP_FLAG() (WDT->CTL = (WDT->CTL & ~(WDT_CTL_RSTF_Msk | WDT_CTL_IF_Msk | WDT_CTL_TOF_Msk)) | WDT_CTL_WKF_Msk)

/** 
  * @brief This macro indicate WDT time-out to reset system or not.
  * @return WDT reset system or not
  * @retval 0 WDT did not cause system reset
  * @retval 1 WDT caused system reset 
  * \hideinitializer
  */
#define WDT_GET_RESET_FLAG() (WDT->CTL & WDT_CTL_RSTF_Msk ? 1 : 0)

/** 
  * @brief This macro indicate WDT time-out evnet occurred or not.
  * @return WDT time-out evnet occurred or not
  * @retval 0 WDT time-out evnet did not occur
  * @retval 1 WDT time-out evnet occurred 
  * \hideinitializer
  */
#define WDT_GET_TIMEOUT_FLAG() (WDT->CTL & WDT_CTL_TOF_Msk ? 1 : 0)

/** 
  * @brief This macro indicate WDT time-out interrupt occurred or not.
  * @return WDT time-out interrupt occurred or not
  * @retval 0 WDT time-out interrupt did not occur
  * @retval 1 WDT time-out interrupt occurred 
  * \hideinitializer
  */
#define WDT_GET_TIMEOUT_INT_FLAG() (WDT->CTL & WDT_CTL_IF_Msk ? 1 : 0)

/** 
  * @brief This macro indicate WDT time-out waked system up or not
  * @return WDT time-out waked system up or not
  * @retval 0 WDT did not wake up system
  * @retval 1 WDT waked up system
  * \hideinitializer
  */
#define WDT_GET_TIMEOUT_WAKEUP_FLAG() (WDT->CTL & WDT_CTL_WKF_Msk ? 1 : 0)

/** 
  * @brief This macro is used to reset 18-bit WDT counter.
  * @details If WDT is activated and enabled to reset system, software must reset WDT counter 
  *  before WDT time-out plus reset delay reached. Or WDT generate a reset signal.
  * \hideinitializer
  */
#define WDT_RESET_COUNTER() (WDT->CTL  = ((WDT->CTL & ~(WDT_CTL_IF_Msk | WDT_CTL_WKF_Msk | WDT_CTL_RSTF_Msk)) | WDT_CTL_RSTCNT_Msk | WDT_CTL_TOF_Msk))


/** 
  * @brief This function clear WDT time-out reset system flag. 
  * \hideinitializer
  */
__STATIC_INLINE void WDT_ClearResetFlag(void)
{
    WDT->CTL |= WDT_CTL_RSTF_Msk;
}

/** 
  * @brief This function clear WDT time-out flag. 
  * \hideinitializer
  */
__STATIC_INLINE void WDT_ClearTimeoutFlag(void)
{
    WDT->CTL |= WDT_CTL_TOF_Msk;
}

/** 
  * @brief This function clear WDT time-out interrupt flag. 
  * \hideinitializer
  */
__STATIC_INLINE void WDT_ClearTimeoutIntFlag(void)
{
    WDT->CTL |= WDT_CTL_IF_Msk;
}

/** 
  * @brief This function clear WDT time-out wake-up system flag. 
  * \hideinitializer
  */
__STATIC_INLINE void WDT_ClearTimeoutWakeupFlag(void)
{
    WDT->CTL |= WDT_CTL_WKF_Msk;
}

/** 
  * @brief This function indicate WDT time-out to reset system or not.
  * @return WDT reset system or not
  * @retval false WDT did not cause system reset
  * @retval true  WDT caused system reset 
  * \hideinitializer
  */
__STATIC_INLINE bool WDT_GetResetFlag(void)
{
    return (bool)(WDT->CTL & WDT_CTL_RSTF_Msk);
}

/** 
  * @brief This function indicate WDT time-out evnet occurred or not.
  * @return WDT time-out evnet occurred or not
  * @retval false WDT time-out evnet did not occur
  * @retval true  WDT time-out evnet occurred 
  * \hideinitializer
  */
__STATIC_INLINE bool WDT_GetTimeoutFlag(void)
{
    return (bool)((WDT->CTL & WDT_CTL_TOF_Msk)>>WDT_CTL_TOF_Pos);
}

/** 
  * @brief This function indicate WDT time-out interrupt occurred or not.
  * @return WDT time-out interrupt occurred or not
  * @retval false WDT time-out interrupt did not occur
  * @retval true  WDT time-out interrupt occurred 
  * \hideinitializer
  */
__STATIC_INLINE bool WDT_GetTimeoutIntFlag(void)
{
    return (bool)(WDT->CTL & WDT_CTL_IF_Msk);
}

/** 
  * @brief This function indicate WDT time-out waked system up or not
  * @return WDT time-out waked system up or not
  * @retval false WDT did not wake up system
  * @retval true  WDT waked up system
  * \hideinitializer
  */
__STATIC_INLINE bool WDT_GetTimeoutWakeupFlag(void)
{
    return (bool)(WDT->CTL & WDT_CTL_WKF_Msk);
}

/** 
  * @brief This function is used to reset 18-bit WDT counter.
  * @details If WDT is activated and enabled to reset system, software must reset WDT counter 
  *  before WDT time-out plus reset delay reached. Or WDT generate a reset signal.
  * \hideinitializer
  */
__STATIC_INLINE void WDT_ResetCounter(void)
{
    WDT->CTL |= WDT_CTL_RSTCNT_Msk;
}

void  WDT_Open(WDT_TimeoutDef u32TimeoutInterval,
                  WDT_ResetDelayDef u32ResetDelay,
                  uint32_t u32EnableReset,
                  uint32_t u32EnableWakeup);  
void WDT_Close(void);
void WDT_EnableInt(void);
void WDT_DisableInt(void);

void WDT_SelectClockSource(uint32_t u32clksel);

/*@}*/ /* end of group PN501_WDT_EXPORTED_FUNCTIONS */



#ifdef __cplusplus
}
#endif

#endif //__WDT_H

