/*******************************************************************************
 * @note Copyright (C) 2020 Shanghai Panchip Microelectronics Co., Ltd.
 * All rights reserved.
 *
 * @file rf.c
 * @brief PAN2025 series RF driver source file.
 *
 * @history - V0.01, 2020-03-18, yangyaming, first implementation.
*******************************************************************************/
#include "rf.h"
#include "fmc.h"
#include "driver_delay.h"

static uint8_t addr[5]=TX_ADDR_DEF;
/**
  * @brief Init RF and Modemn
  * @param[in] void
  * @return None
  */
void RF_Init(void)
{
	uint16_t idx = 0;
	uint16_t two_point_val = 0;
	uint32_t chip_version_flag;
	
	chip_version_flag = flash_ReadInfo(FMC_INFO_PAN2025_VERSION_ADDR);
    
	CE_LOW;
	RF_ClockInit();

	RF_ClearFIFO();
	RF_ClearStatus();   
		
	/* Enable rx channel 0 */
	RF->EN_RXADDR = 0x01;
	
	/* config rx/tx addr witdth*/
	RF->SETUP_AW = ADDR_WITDTH;
	
	/* config default channel */
	RF->RF_CH = 0x6e;

	RF_SetAddr(RF->TX_ADDR, addr, sizeof(addr));
	
	RF_SetAddr(RF->RX_ADDR_P0,addr,sizeof(addr));
	
//    RF->PN006_SET_0 = 0x45;
//    RF->PN006_SET_1 = 0x7e;     
//    RF->PN006_SET_2 = 0x1b;
//    RF->PN006_SET_3 = 0xff;
    RF->PN006_SET_4 = 0x77;     //default value 0x78
    RF->PN006_SET_5 = 0x64;     //default value 0xfc
    RF->PN006_SET_6 = 0x72;     //default value 0x73
//    RF->PN006_SET_7 = 0x00;

//    RF->DEMOD_CAL_0 = 0x04;     //default value 0x05    down config
//    RF->DEMOD_CAL_1 = 0x41;     
//    RF->DEMOD_CAL_2 = 0x7c;
//    RF->DEMOD_CAL_3 = 0x1f;
//    RF->DEMOD_CAL_4 = 0x00;
    RF->DEMOD_CAL_5 = 0x02;
//    RF->DEMOD_CAL_6 = 0x0;
    RF->DEMOD_CAL_7 = 0x56;     //default value 0x20  
//    RF->DEMOD_CAL_8 = 0x5b;     //default value 0x5a      down config
	

    RF->RF_CAL2_0 = 0x08;
//    RF->RF_CAL2_1 = 0xc2;
    RF->RF_CAL2_2 = 0xdf;       //default value 0xff
    RF->RF_CAL2_3 = 0x9f;       //default value 0x97
    RF->RF_CAL2_4 = 0x8b;       //default value 0xab
    RF->RF_CAL2_5 = 0x62;       //default value 0x42
//    RF->RF_CAL2_6 = 0xff;
    RF->RF_CAL2_7 = 0x7f;       //default value 0x64
    RF->RF_CAL2_8 = 0xcf;
    RF->RF_CAL2_9 = 0x17;       //default value 0x24

		RF->DEM_CAL2_0 = 0xb1;      //default value 0x91
//    RF->DEM_CAL2_1 = 0xcb;
//    RF->DEM_CAL2_2 = 0xd3;
//    RF->DEM_CAL2_3 = 0xba;
//    RF->DEM_CAL2_4 = 0x78;
//    RF->DEM_CAL2_5 = 0x88;
//    RF->DEM_CAL2_6 = 0x0b;
//    RF->DEM_CAL2_7 = 0xa8;
//    RF->DEM_CAL2_8 = 0xaa;
	
//    RF->DYNPD = 0x00;
    
    
//    RF->FEATURE = 0x00;


//    RF->RF_CAL_0 = 0x80;        
    RF->RF_CAL_1 = 0x02;
    RF->RF_CAL_2 = 0xa0;
//    RF->RF_CAL_3 = 0x88;
//    RF->RF_CAL_4 = 0x71;
    RF->RF_CAL_5 = 0xbb;
    RF->RF_CAL_6 = 0xa1;        //default value 0x61
    RF->RF_CAL_7 = 0x80;        //default value 0x54
    RF->RF_CAL_8 = 0x53;        //default value 0x61
    RF->RF_CAL_9 = 0xf0;        //default value 0x80

	//    RF->BB_CAL_0 = 0x50;        //default value 0x0a  down config
    RF->BB_CAL_1 = 0x7b;        //default value 0x6d
    RF->BB_CAL_2 = 0xe4;        //default value 0x24
//    RF->BB_CAL_3 = 0x24;
//    RF->BB_CAL_4 = 0x4e;
//    RF->BB_CAL_5 = 0x20;
    RF->BB_CAL_6 = 0xff;        //default value 0x7f
    RF->BB_CAL_7 = 0x7c;        //default value 0x9b  down config
//    RF->BB_CAL_8 = 0xfe;        //default value 0xe4  down config
    RF->BB_CAL_9 = 0x01;        //default value 0x00

    RF->RX_PW_P0 = RX_PAYLOAD_LEN;      //payload

	
#if(TRANSMIT_TYPE == TRANS_ENHANCED_MODE)  

    RF->SETUP_RETR = 0x03;    //  2 retrans 	
    RF->EN_AA = 0x01;    // Enable Auto.Ack:Pipe0
    //active command
    RF->RF_CMD = ACTIVATE;

	#if ((FEATURE_MODE & DYN_MSK) == DYN_MSK)
        RF->FEATURE |= FEATURES_EN_DPL_Msk;
        RF->DYNPD |= DYNPD_DPL_P0_Msk;
    #else
        RF->FEATURE &= ~FEATURES_EN_DPL_Msk;
        RF->DYNPD &= ~DYNPD_DPL_P0_Msk;
    #endif
    
	#if ((FEATURE_MODE & ACK_WITH_PAYLOAD_MSK) == ACK_WITH_PAYLOAD_MSK)
        RF->FEATURE |= FEATURES_EN_ACK_PAY_Msk;    
    #else
        RF->FEATURE &= ~FEATURES_EN_ACK_PAY_Msk;
    #endif
    
	#if ((FEATURE_MODE & NOACK_MSK) == NOACK_MSK)
        RF->FEATURE |= FEATURES_EN_NOACK_Msk;   
    #else
        RF->FEATURE &= ~FEATURES_EN_NOACK_Msk;
    #endif
    
	
		
#elif(TRANSMIT_TYPE == TRANS_NORMAL_MODE)                                                                
    RF->SETUP_RETR = 0x00;						// Disable retrans	
    RF->EN_AA = 0x00;							// Disable AutoAck 
#endif


#if (RF_DR == RF_DR_1M_DSSS_DSB)
	RF_SetDataRate(DR_1M,DSSS_DSB);
    RF->BB_CAL_0 = 0x54;    //RX_ACK_TIME
		RF->DEMOD_CAL_0 &= ~FRA_SPI_IN_EN_Msk;
		RF->DEMOD_CAL_1 = 0x41;
    RF->DEMOD_CAL_8 = 0x5b;
    
    RF->BB_CAL_7 = 0x7c;    //??DAC_H
    RF->BB_CAL_8 = 0xb3;    //??DAC_L&GC ea
#elif(RF_DR == RF_DR_1M_DSSS_EN)                                                                
	RF_SetDataRate(DR_1M,DSSS_EN);
    RF->BB_CAL_0 = 0x55;    //RX_ACK_TIME
    RF->DEMOD_CAL_0 = 0x04;
    RF->DEMOD_CAL_1 = 0x41;
    RF->DEMOD_CAL_8 = 0x5b;
    
    RF->BB_CAL_7 = 0x7c;    //DAC_H
    RF->BB_CAL_8 = 0xeb;    //DAC_L&GC     
#elif(RF_DR == RF_DR_2M)                                                                
	RF_SetDataRate(DR_2M,DSSS_DSB);
    RF->BB_CAL_0 = 0x5F;    //RX_ACK_TIME
    RF->DEMOD_CAL_0 = 0x04;
    RF->DEMOD_CAL_1 = 0x40;
    RF->DEMOD_CAL_8 = 0x70;
    
    RF->BB_CAL_7 = 0x9c;    //DAC_H
    RF->BB_CAL_8 = 0xfa;    //DAC_L&GC FB
#elif(RF_DR == RF_DR_250K)                                                                
	RF_SetDataRate(DR_250K,DSSS_DSB);
    RF->BB_CAL_0 = 0x54;    //RX_ACK_TIME
    RF->DEMOD_CAL_0 = 0x04;
    RF->DEMOD_CAL_1 = 0x41;
    RF->DEMOD_CAL_8 = 0x47;
    
    RF->BB_CAL_7 = 0x5c;    //DAC_H
    RF->BB_CAL_8 = 0xF5;    //DAC_L&GC ea    
#endif  
	RF_WhiteningConfig( ENABLE);

//	//Tx Mode
	RF->CONFIG = 0x8E;	
	
	CE_HIGH;
	/* Filter calibration */
	RF->RF_CAL2_4 = 0xAb;
	
	for(idx = 0; idx < 5; idx++)
    { 
        RF->DEMOD_CAL_7 =  0X56;
        CLK_SysTickDelay(100);  
        RF->DEMOD_CAL_7 =  0X16;	
        CLK_SysTickDelay(100); 
        while(!(RF->DATAOUT_3 & (0x1 << 3))); 
		CLK_SysTickDelay(1);
        two_point_val += (RF->DATAOUT_2 & 0x0f);   
    }
	two_point_val=(two_point_val/5);
	RF->DEMOD_CAL_7 = (0x00|(two_point_val<<2));
    RF_VccodeRead();
}

void RF_ClockInit(void)
{
    CLK_AHBPeriphClockCmd(CLK_AHBPeriph_RF,ENABLE);
    
    ANAC->LP_ANA_CTRL0 |= EN_LDO_Msk;              //EN_LDO
    ANAC->LP_FL_CTRL1 |= EN_LDO_DPLL_Msk;   //EN_LDO_DPLL
    CLK_Wait3vSyncReady();  
    //dpll select
    ANAC->LP_MISC |= DPLL_CLK_SEL_Msk;
    CLK_Wait3vSyncReady();
    ANAC->LP_FL_CTRL1 |= EN_DPLL_Msk;
    CLK_Wait3vSyncReady();
}

void PowerSet(void)
{
	uint16_t tmpreg = 0;
	uint16_t g_power_paramer[]=RF_POWER;
	
	//Configuration Power register: BB_CAL_9 | BB_CAL_8 | PN006_SET_6 | PN006_SET_3 | BB_CAL_6
	tmpreg = RF->BB_CAL_9;
	tmpreg &= 0xfe;
	tmpreg |= g_power_paramer[0];
	RF->BB_CAL_9 = tmpreg;
	
	tmpreg = RF->BB_CAL_8;
	tmpreg &= 0x3f;
	tmpreg |= g_power_paramer[1];
	RF->BB_CAL_8 = tmpreg;
	
	tmpreg = RF->PN006_SET_6;
	tmpreg &= 0x8f;
	tmpreg |= g_power_paramer[2];
	RF->PN006_SET_6 = tmpreg;
	
	tmpreg = RF->PN006_SET_3;
	tmpreg &= 0x03;
	tmpreg |= g_power_paramer[3];
	RF->PN006_SET_3 = tmpreg;

	tmpreg = RF->BB_CAL_6;
	tmpreg &= 0xe7;
	tmpreg |= g_power_paramer[4];
	RF->BB_CAL_6 = tmpreg;
}

/**
  * @brief Set RF into TX mode
  * @param[in] void
  * @return None
  */
void RF_TxMode(void)
{
    CE_LOW;
		delay_us(1);
    RF->CONFIG = 0x8E;
    CE_HIGH;
}



/**
  * @brief Set RF into RX mode
  * @param[in] void
  * @return None
  */
void RF_RxMode(void)
{
    CE_LOW;
    // RFóRXģʽ
    RF->CONFIG = 0x8F;							
    // Set CE pin high ʼ
    CE_HIGH;							
}

/**
  * @brief read RF IRQ status
  * @param[in] void
  * @return 3-bits status
  */
uint8_t RF_GetStatus(void)
{
    //ȡRF״̬ 
    return RF->IRQ_STATUS & 0x70;
}

/**
  * @brief clear RF IRQ
  * @param[in] void
  * @return None
  */
void RF_ClearStatus(void)
{
    //RFIRQ־ 
    RF->IRQ_STATUS = 0x70;						
}


/**
  * @brief clear RF TX/RX FIFO
  * @param[in] void
  * @return None
  */
void RF_ClearFIFO(void)
{
    //RF  TX FIFO	
    RF->RF_CMD = FLUSH_TX;	    
    //RF  RX FIFO	    
    RF->RF_CMD = FLUSH_RX;                                                  		
}

uint8_t rf_ch[8] = {5,15,25,35,45,55,65,75};
uint8_t RfVcoCodeBuff[8];

void RF_VccodeRead(void)
{
    uint8_t i;
	RF_RxMode();
	CLK_SysTickDelay(1000);

	for(i=0;i<8;i++)
	{
		RF->RF_CH = rf_ch[i];
		CLK_SysTickDelay(5);
		while(!(RF->DATAOUT_3 & (0x1 << 2))); 
		CLK_SysTickDelay(1);
		RfVcoCodeBuff[i] = RF->DATAOUT_1>>4;		
	}
	//RF->DEMOD_CAL_1 = 0x01;		//off auto vco cal
    RF->DEMOD_CAL_1 &= ~0x40;
	RF->RF_CAL_4 = 0x75;			//open manu vco code
	RF->RF_CE = 0; 
}

/**
  * @brief Set RF TX/RX channel
  * @param[in] chn RF communication channel
  * @return None
  */
void RF_SetChannel( uint8_t chn)
{
    if(chn < 10){
		RF->RF_CAL_3 = (RfVcoCodeBuff[0]) | (RfVcoCodeBuff[0] << 4);
	}else if(chn < 20){
        RF->RF_CAL_3 = (RfVcoCodeBuff[1]) | (RfVcoCodeBuff[1] << 4);
    }else if(chn < 30){
        RF->RF_CAL_3 = (RfVcoCodeBuff[2]) | (RfVcoCodeBuff[2] << 4);  
    }else if(chn < 40){
        RF->RF_CAL_3 = (RfVcoCodeBuff[3]) | (RfVcoCodeBuff[3] << 4);
    }else if(chn < 50){
        RF->RF_CAL_3 = (RfVcoCodeBuff[4]) | (RfVcoCodeBuff[4] << 4);
    }else if(chn < 60){
        RF->RF_CAL_3 = (RfVcoCodeBuff[5]) | (RfVcoCodeBuff[5] << 4);  
    }else if(chn < 70){
        RF->RF_CAL_3 = (RfVcoCodeBuff[6]) | (RfVcoCodeBuff[6] << 4);   
    }else if(chn < 83){
        RF->RF_CAL_3 = (RfVcoCodeBuff[7]) | (RfVcoCodeBuff[7] << 4); 
    }
    RF->RF_CH = chn;
}
/**
  * @brief Set RF desired RX data length
  * @param[in] length desired RX data length
  * @return None
  */
void RF_SetRxLength( uint8_t length)
{    
    RF->RX_PW_P0 = length;
    
}
/**
  * @brief Get RF desired RX data length
  * @param[in] length desired RX data length
  * @return None
  */
uint32_t RF_GetRxLength()
{
    return RF->RX_RPL_WID;
}

/**
  * @brief Set Data rate
  * @param[in] rate is RF communication data rate. Including :
  *             - \ref DR_1M
  *             - \ref DR_2M
  *             - \ref DR_250K
  * @param[in] dsss dsss function enable or not. Including:
  *             - \ref DSSS_EN
  *             - \ref DSSS_DSB
  * @return None
  */
void RF_SetDataRate( uint8_t rate,uint8_t dsss)
{    
    RF->RF_SETUP = rate;
    
    if(dsss){
        //DSSS Enable
        RF->PN006_SET_2 = 0x9b;
        RF->BB_CAL_4 = 0x4c;
    }
}
/**
  * @brief Set data Whitening
  * @param[in] scr scr function enable or not. Including:
  *             - \ref SCR_EN
  *             - \ref SCR_DSB
  * @return None
  */
void RF_WhiteningConfig( FunctionalState NewState)
{
    if(NewState){        
        RF->DEMOD_CAL_0 |= SCR_EN_Msk;
    }
    else{
        RF->DEMOD_CAL_0 &= ~SCR_EN_Msk;
    }
}

/**
  * @brief Read Data from RF Fifo
  * @param[in] pBuf    Pointer which points to a block of memory
  * @param[in] length  Data length
  * @return None
  */
void RF_ReadBuf( uint8_t cmd, unsigned char *pBuf,  uint8_t length)
{
    uint8_t i;  
    
    RF->RF_CMD = cmd;
    for(i = 0; i < length; i++){
    	pBuf[i] = RF->RF_FIFO;
    }       
    RF->RF_CMD = NOP;

}

/**
  * @brief Write Data to RF Fifo
  * @param[in] pBuf    Pointer which points to a block of memory
  * @param[in] length  Data length
  * @return None
  */
void RF_WriteBuf(uint8_t cmd,uint8_t *pBuf, uint8_t length)
{
    uint8_t i;

    RF->RF_CMD = cmd;
    
  	for(i = 0; i < length; i++){
        RF->RF_FIFO = *pBuf++;
    }  
    RF->RF_CMD = NOP;

    CE_HIGH;

}

void RF_SetAddr(volatile uint32_t* reg, uint8_t* rf_addr_array, uint8_t addr_len)
{
    uint8_t i;
    for(i = 0; i < addr_len; i++){
        reg[i] = rf_addr_array[i];
    }
}

void RF_ReadAddr(volatile uint32_t* reg, uint8_t* rf_addr_array, uint8_t addr_len)
{
    uint8_t i;
    for(i = 0; i < addr_len; i++){
        rf_addr_array[i] = reg[i];
    }
}
/**
  * @brief Enter Single Carrier Mode
  * @param[in] channel   Single Carrier Mode channel
  * @return None
  */

/**
  * @brief Enter Single Carrier Mode
  * @param[in] channel   Single Carrier Mode channel
  * @return None
  */
void RF_Carrier( uint8_t channel)
{
    RF_ClockInit();
    PowerSet();
    RF->RF_CH = 110;
    RF->PN006_SET_1 = 0x7e;
    RF->PN006_SET_3 = 0xFF;
    RF->PN006_SET_6 = 0X72;
    RF->DEMOD_CAL_0 = 0x1F;
    RF->DEMOD_CAL_1 = 0x49;
    RF->DEMOD_CAL_5 = 0x12;
    RF->RF_CAL2_0 = 0x08;
    RF->RF_CAL2_2 = 0xff;
    RF->RF_CAL2_5 = 0x51;
    RF->RF_CAL2_8 = 0xFF;
    RF->RF_CAL2_9 = 0x3f;
    RF->RF_CAL_1 = 0x02;
    RF->RF_CAL_2 = 0xa0;
    RF->RF_CAL_3 = 0x88;
    RF->RF_CAL_4 = 0x71;
    RF->RF_CAL_5 = 0xb8;
    RF->RF_CAL_6 = 0xa1;
    RF->RF_CAL_7 = 0x54;
    RF->RF_CAL_8 = 0x55;
    RF->RF_CAL_9 = 0xf0;
    RF->BB_CAL_6 = 0xff;
    RF->BB_CAL_7 = 0x7b;
    RF->BB_CAL_8 = 0xaa;
    RF->BB_CAL_9 = 0x00;
    //Ҫ
    RF->DEMOD_CAL_5 = 0x12;
    CLK_SysTickDelay(1000);
    RF->RF_CH = channel;
    CE_HIGH;
}



#define BASEBAND_PARAM9_PA_ICTR_Pos         (0)
#define BASEBAND_PARAM9_PA_ICTR_Msk         (0x1<<BASEBAND_PARAM9_PA_ICTR_Pos)
#define BASEBAND_PARAM8_PA_ICTR_Pos         (6)
#define BASEBAND_PARAM8_PA_ICTR_Msk         (0x3<<BASEBAND_PARAM8_PA_ICTR_Pos)
#define DEMODULATOR_CFG6_PA2ST_BC_Pos       (4)
#define DEMODULATOR_CFG6_PA2ST_BC_Msk       (0x7<<DEMODULATOR_CFG6_PA2ST_BC_Pos)
#define DEMODULATOR_CFG3_PA2ST_RAMP_Pos     (2)
#define DEMODULATOR_CFG3_PA2ST_RAMP_Msk     (0x7<<DEMODULATOR_CFG3_PA2ST_RAMP_Pos)
#define DEMODULATOR_CFG3_PA1ST_GC_Pos       (5)
#define DEMODULATOR_CFG3_PA1ST_GC_Msk       (0x7<<DEMODULATOR_CFG3_PA1ST_GC_Pos)
#define BASEBAND_PARAM6_PA1ST_ISEL_Pos      (3)
#define BASEBAND_PARAM6_PA1ST_ISEL_Msk      (0x3<<BASEBAND_PARAM6_PA1ST_ISEL_Pos)
void PowerSetBackup(uint8_t power)
{
    const uint8_t bbCal8Value[]       = {0x02,0x02,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
    const uint8_t demodCfg6Value[]    = {0x07,0x04,0x03,0x07,0x06,0x05,0x03,0x02,0x02,0x02,0x01,0x1,0x00,0x00};
    const uint8_t demodCfg3RampValue[]= {0x07,0x07,0x06,0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x06,0x04};
    const uint8_t demodCfg3GcValue[]  = {0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x07,0x02,0x04,0x07,0x02,0x02,0x00};
    const uint8_t bbCal6Value[]       = {0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x00,0x03,0x03,0x03,0x03}; 
    
    RF->BB_CAL_9    = (RF->BB_CAL_9     & ~BASEBAND_PARAM9_PA_ICTR_Msk); 
    RF->BB_CAL_8    = ((RF->BB_CAL_8    & ~BASEBAND_PARAM8_PA_ICTR_Msk  ) | (bbCal8Value[power-1]<<BASEBAND_PARAM8_PA_ICTR_Pos)); 
    RF->PN006_SET_6 = ((RF->PN006_SET_6 & ~DEMODULATOR_CFG6_PA2ST_BC_Msk) | (demodCfg6Value[power-1]<<DEMODULATOR_CFG6_PA2ST_BC_Pos));
    RF->PN006_SET_3 &=  ~(DEMODULATOR_CFG3_PA2ST_RAMP_Msk | DEMODULATOR_CFG3_PA1ST_GC_Msk);
    RF->PN006_SET_3 |=((demodCfg3RampValue[power-1]<<DEMODULATOR_CFG3_PA2ST_RAMP_Pos) | (demodCfg3GcValue[power-1]<<DEMODULATOR_CFG3_PA1ST_GC_Pos));
    RF->BB_CAL_6    = ((RF->BB_CAL_6    & ~BASEBAND_PARAM6_PA1ST_ISEL_Msk) | (bbCal6Value[power]));
}
