#include "use_ble.h"
#include "rf.h"
#include <string.h>

const static uint8_t whitening_map[3][42] = 
{
	// 37
	{
		0x8D, 0xD2, 0x57, 0xA1, 0x3D,
		0xA7, 0x66, 0xB0, 0x75, 0x31,
		0x11, 0x48,	0x96, 0x77, 0xF8,
		0xE3, 0x46, 0xE9, 0xAB, 0xD0,
		0x9E, 0x53, 0x33, 0xD8,	0xBA,
		0x98, 0x08, 0x24, 0xCB, 0x3B,
		0xFC, 0x71, 0xA3, 0xF4, 0x55,
		0x68, 0xCF, 0xA9, 0x19, 0x6C,
		0x5D, 0x4C
	},
	
	//38
	{
		0xD6, 0xC5, 0x44, 0x20, 0x59,
		0xDE, 0xE1, 0x8F, 0x1B, 0xA5,
		0xAF, 0x42, 0x7B, 0x4E, 0xCD,
		0x60, 0xEB, 0x62, 0x22, 0x90,
		0x2C, 0xEF, 0xF0, 0xC7, 0x8D,
		0xD2, 0x57, 0xA1, 0x3D, 0xA7,
		0x66, 0xB0, 0x75, 0x31, 0x11,
		0x48, 0x96, 0x77 ,0xF8, 0xE3,
		0x46, 0xE9
	},
	
	//39
	{
		0x1F, 0x37, 0x4A, 0x5F, 0x85,
		0xF6, 0x9C, 0x9A, 0xC1, 0xD6,
		0xC5, 0x44, 0x20, 0x59, 0xDE,
		0xE1, 0x8F, 0x1B, 0xA5, 0xAF,
		0x42, 0x7B, 0x4E, 0xCD, 0x60,
		0xEB, 0x62, 0x22, 0x90, 0x2C,
		0xEF, 0xF0, 0xC7, 0x8D, 0xD2,
		0x57, 0xA1, 0x3D, 0xA7, 0x66,
		0xB0, 0x75
	}
};

//BLE㲥֡ṹ
//Adv Data ɲͬAD Structure 
/*****************************************************************************
  Preamble * Access Address*               PDU                      *   CRC  *
******************************************************************************
           *               * PDU Header *  Adv Address *  Adv Data  *        *
   1byte   *    4bytes     ****************************************** 3bytes *
           *               *   2bytes   *     6bytes   *  0-31 bytes*        *
******************************************************************************/


const static uint8_t ble_nonconn_buffer[BLE_FIX_FRAME_LENGTH] = 
{
        0xAA,                    //0xAA:preamble ޸
        0xD6, 0xBE, 0x89, 0x8E,  //0xD6, 0xBE, 0x89, 0x8E:Access Address ޸	                                 								
	0x42, 0xFF,              // 0x42, 0xFFPDU Header   0xFFΪPDUЧ  Ҫݺ Manufacturer Specific Data޸ 
        0x66, 0x55, 0x44, 0x33, 0x22, 0x11, //0x66-0x11:Adv Address û޸
        //ADV Data
	0x05, 0x09, 'T', 'E', 'M', 'P',     // 	AD Structure 1:豸֣û޸ ֳȷ仯ע޸ BLE_FIX_FRAME_LENGTHֵ
        0x02, 0x01, 0x1A,                   // 	AD Structure 2:Flags for discoverability. һ㲻޸															
	0xFF, 0xFF, 0xF0, 0xFF	            // 	AD Structure 3:Manufacturer Specific Data ûЭݷڸýṹ,һ0xFFΪֽ֡ Ҫʵ޸	 												
};

uint8_t invert_8(uint8_t data);
uint32_t invert_32(uint32_t data);
uint32_t check_crc24(const uint8_t *data, uint8_t length);

void ble_craft_packet_noconn(uint8_t *data, uint8_t length, uint8_t channel_index, uint8_t *ble_buffer)
{
    uint8_t rf_tx_buffer[47];
    memcpy(rf_tx_buffer, ble_nonconn_buffer, BLE_FIX_FRAME_LENGTH);
    memcpy(rf_tx_buffer + BLE_FIX_FRAME_LENGTH, data, length);
    
    rf_tx_buffer[PDU_LENGTH_INDEX] = 19 + length; //PDU  19=Adv Address(6)+D Structure 1(6)+AD Structure 2(3)+ AD Structure 3(4)+length
    rf_tx_buffer[MANUFACTURER_DATD_LENGTH_INDEX] = 3 + length;
    
    uint32_t calc_crc = check_crc24(rf_tx_buffer + 5, 21 + length);//CRCУ鷶ΧΪPDU  21=PDU Header(2)+Adv Address(6)+D Structure 1(6)+AD Structure 2(3)+ AD Structure 3(4)+length
    
    rf_tx_buffer[BLE_FIX_FRAME_LENGTH + length + 0] = calc_crc;
    rf_tx_buffer[BLE_FIX_FRAME_LENGTH + length + 1] = calc_crc >> 8;
    rf_tx_buffer[BLE_FIX_FRAME_LENGTH + length + 2] = calc_crc >> 16;
    
    //׻ķΧΪPDU+CRC    24=PDU Header(2)+Adv Address(6)+D Structure 1(6)+AD Structure 2(3)+ AD Structure 3(4)+length+CRC(3)
    for (uint8_t i = 0; i < 24 + length; i++)
    {
            rf_tx_buffer[5 + i] ^= whitening_map[channel_index - 37][i];
    }
    
    memcpy(ble_buffer, rf_tx_buffer,BLE_FIX_FRAME_LENGTH+BLE_CRC_LENGTH+ length);  //29=preamble(1)+Access Address(4)+PDU Header(2)+Adv Address(6)+D Structure 1(6)+AD Structure 2(3)+ AD Structure 3(4)+length+CRC(3)
}

uint32_t check_crc24(const uint8_t *data, uint8_t length)
{
	uint32_t crc = 0x55555500;
	uint32_t poly = 0x000065B00;
	uint32_t input_byte = 0;
	
	for (uint8_t i = 0; i < length; i++)
	{
		input_byte = invert_8(data[i]);
		
		crc ^= (input_byte << 24);
		
		for (uint8_t j = 0; j < 8; j++)
		{
			if (crc & 0x80000000)
			{
				crc = (crc << 1) ^ poly;
			}
			else
			{
				crc = (crc << 1);
			}
		}
	}
	
	crc = invert_32(crc);
	return crc;
}

uint8_t invert_8(uint8_t data)
{
	uint8_t temp = 0;
	
	for (uint8_t i = 0; i < 8; i++)
	{
		if (data & (1 << i))
		{
			temp |= 1 << (7 - i);
		}
	}
	
	return temp;
}

uint32_t invert_32(uint32_t data)
{
	register uint32_t temp = 0;

	for (uint8_t i = 0; i < 32; i++)
	{
		if (data & (1UL << i))
		{
			temp |= 1UL << (31 - i);
		}
		else
		{
			temp = temp;
		}
	}
	
	return temp;
}

 
	/* transmit count*/
extern volatile uint32_t g_tx_flag;
/* receive count*/
extern volatile uint32_t g_rx_cnt;

/* receive/transmit buf*/
extern uint8_t rf_tx_payload[TX_PAYLOAD_LEN];
extern uint8_t rf_rx_payload[RX_PAYLOAD_LEN];

// ̶ͳݲ
void rf_tx_send_data(uint8_t *pBuff)
{
    // Valid Length = 23 + data length
    // Max Length = 47
    uint8_t rf_buffer[47];
    
    uint8_t temp_length =CONF_PAYLOAD_WIDTH;  //ݳ
    
    // ble hannel index = 37
    // freq = 2402MHz
    ble_craft_packet_noconn(pBuff, temp_length, 37, rf_buffer);
    RF_SetChannel(02);
		RF_WriteBuf(W_TX_PAYLOAD,rf_buffer, BLE_FIX_FRAME_LENGTH + temp_length + 3);
    while(!g_tx_flag);
		g_tx_flag = 0;
		RF_ClearFIFO();

    // ble hannel index = 38
    // freq = 2426MHz		
    ble_craft_packet_noconn(pBuff, temp_length, 38, rf_buffer);
    RF_SetChannel(26);
    RF_WriteBuf(W_TX_PAYLOAD,rf_buffer,  BLE_FIX_FRAME_LENGTH + temp_length + 3);
    while(!g_tx_flag);
		g_tx_flag = 0;
		RF_ClearFIFO();
    
    // ble hannel index = 39
    // freq = 2480MHz
    ble_craft_packet_noconn(pBuff, temp_length, 39, rf_buffer);
    RF_SetChannel(80);
    RF_WriteBuf(W_TX_PAYLOAD,rf_buffer,  BLE_FIX_FRAME_LENGTH + temp_length + 3);
    while(!g_tx_flag);
		g_tx_flag = 0;
		RF_ClearFIFO();
 
}


// ̶8bytesݲ
uint8_t rf_tx_rev_data(void)
{
		uint8_t i;
    if (g_rx_cnt)
    {
			//յ  rf_rx_payload
			printf("rx_data\r\n");
			for(i=0;i<CONF_PAYLOAD_WIDTH;i++)printf(" %x ",rf_rx_payload[i]);
			printf("\r\n");
			g_rx_cnt = 0;
		 	RF_ClearFIFO(); 
			return 1;			
    }
		return 0;
}

uint8_t tx_rx_address[] = CONF_DEFAULT_ADDR;
void ble_rf_init(uint8_t whitening_enable)
{
	RF_SetChannel(CONF_DEFAULE_CNANNEL);   // ýŵ
	
	RF->RX_PW_P0 = CONF_PAYLOAD_WIDTH;      // ýݿȣPayload
	
	RF->SETUP_AW = sizeof(tx_rx_address)-2;
	RF_SetAddr(RF->TX_ADDR, tx_rx_address, sizeof(tx_rx_address));  // ÷͵ַ
	RF_SetAddr(RF->RX_ADDR_P0,tx_rx_address,sizeof(tx_rx_address)); // ýյַ
	RF->FEATURE |= 0x18;
	RF->DEMOD_CAL_0 &=~0x01;
	RF->DEMOD_CAL_0 |=whitening_enable;
}