/*******************************************************************************
 * @note Copyright (C) 2019 Shanghai Panchip Microelectronics Co., Ltd.
 * All rights reserved.
 *
 * @file user_rf.c
 * @brief 
 *
 * @history - V0.01, 2020-04-14, caixiaohai, first implementation.
*******************************************************************************/
#include "user_rf.h"
#include "driver_delay.h"

/* receive/transmit buf*/
uint8_t rf_tx_payload[TX_PAYLOAD_LEN];
uint8_t rf_rx_payload[RX_PAYLOAD_LEN];
/* transmit count*/
volatile uint32_t g_tx_flag = 0;
/* receive count*/
volatile uint32_t g_rx_cnt = 0;

/**
  * @brief  RF IRQ handle
  * @param  None
  * @retval None
  */
void RF_IRQHandler(void)
{
	uint32_t rx_length = 0;
	
	/* new data arrives in RX FIFO*/
	if(((RF->IRQ_STATUS) & IRQ_RX_DR) == IRQ_RX_DR)
	{
		rx_length = RF_GetRxLength();
		RF_ReadBuf(R_RX_PAYLOAD,rf_rx_payload, rx_length);
		g_rx_cnt++;
		RF->RF_CMD = FLUSH_RX;
	}
	else if(((RF->IRQ_STATUS) & IRQ_TX_DS)== IRQ_TX_DS)
	{
		RF->RF_CMD = FLUSH_TX;	
		g_tx_flag = 1;
	}
	
	RF->IRQ_STATUS = 0x70;	
}

/**
  * @brief  Pack Tx Data
  * @param  None
  * @retval None
  */
static void RF_PackData(uint8_t *pBuf, uint8_t length)
{
    uint8_t i; 
    
    for(i = 0;i < length; i++){
        pBuf[i] = 0x55;
    }
}

/**
  * @brief  rf_tx_busrt_interrupte_test
  * @param  None
  * @retval None
  */
void rf_tx_busrt_interrupte_test(void)
{
	volatile uint32_t cnt = 0;
	RF_Init();
	PowerSet();
	NVIC_EnableIRQ(RF_IRQn);
	RF_TxMode();
	RF_SetChannel(RF_CHANNEL);
	printf("|  rf init sucess                                           |\n");

	RF_PackData(rf_tx_payload,sizeof(rf_tx_payload));
	RF_WriteBuf(W_TX_PAYLOAD,rf_tx_payload, sizeof(rf_tx_payload));
	
	while(1){
		for(cnt = 1; cnt < 1000 ; cnt++)
		{
			while(!g_tx_flag);
			g_tx_flag = 0;
			printf("Tx OK,Send %d packets. \n",cnt);
			delay_us(1000);
			RF_WriteBuf(W_TX_PAYLOAD,rf_tx_payload, sizeof(rf_tx_payload));
		} 
		printf("|  Please press any char to continue                        |\n");
		getchar();		
	}
}

/**
  * @brief  rf_rx_busrt_interrupte_test
  * @param  None
  * @retval None
  */
void rf_rx_busrt_interrupte_test(void)
{	
	RF_Init();
	NVIC_EnableIRQ(RF_IRQn);	
	RF_RxMode();
	RF_SetChannel(RF_CHANNEL);
	printf("|  rf init sucess                                           |\n");
	
	while(1){
		delay_ms(500);
		printf("rx pkts = %d\n",g_rx_cnt);
	}
}

